@@ -1,6 +1,6 @@
 /*
 
-Copyright 2012-2013 Adam Reichold
+Copyright 2012-2014 Adam Reichold
 
 This file is part of qpdfview.
 
@@ -24,19 +24,24 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <QMenu>
 
+class QFileInfo;
+
 #include "global.h"
 
+namespace qpdfview
+{
+
 class BookmarkMenu : public QMenu
 {
     Q_OBJECT
 
 public:
-    BookmarkMenu(const QString& filePath, QWidget* parent = 0);
+    BookmarkMenu(const QFileInfo& fileInfo, QWidget* parent = 0);
 
     void addJumpToPageAction(int page, const QString& label);
     void removeJumpToPageAction(int page);
 
-    QString filePath() const;
+    QString absoluteFilePath() const;
     JumpList jumps() const;
 
 signals:
@@ -61,4 +66,6 @@ private:
 
 };
 
+} // qpdfview
+
 #endif // BOOKMARKMENU_H
