@@ -32,7 +32,12 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <libdjvu/ddjvuapi.h>
 #include <libdjvu/miniexp.h>
 
-static void clearMessageQueue(ddjvu_context_t* context, bool wait)
+namespace
+{
+
+using namespace qpdfview::Model;
+
+void clearMessageQueue(ddjvu_context_t* context, bool wait)
 {
     if(wait)
     {
@@ -52,7 +57,7 @@ static void clearMessageQueue(ddjvu_context_t* context, bool wait)
     }
 }
 
-static void waitForMessageTag(ddjvu_context_t* context, ddjvu_message_tag_t tag)
+void waitForMessageTag(ddjvu_context_t* context, ddjvu_message_tag_t tag)
 {
     ddjvu_message_wait(context);
 
@@ -76,7 +81,245 @@ static void waitForMessageTag(ddjvu_context_t* context, ddjvu_message_tag_t tag)
     }
 }
 
-Model::DjVuPage::DjVuPage(const DjVuDocument* parent, int index, const ddjvu_pageinfo_t& pageinfo) :
+QPainterPath loadLinkBoundary(const QString& type, miniexp_t linkExp, const QSizeF& size)
+{
+    QPainterPath boundary;
+
+    miniexp_t areaExp = miniexp_nth(3, linkExp);
+    const int areaLength = miniexp_length(areaExp);
+
+    if(areaLength == 5 && (type == QLatin1String("rect") || type == QLatin1String("oval")))
+    {
+        QPoint p(miniexp_to_int(miniexp_nth(1, areaExp)), miniexp_to_int(miniexp_nth(2, areaExp)));
+        QSize s(miniexp_to_int(miniexp_nth(3, areaExp)), miniexp_to_int(miniexp_nth(4, areaExp)));
+
+        p.setY(size.height() - s.height() - p.y());
+
+        const QRectF r(p, s);
+
+        if(type == QLatin1String("rect"))
+        {
+            boundary.addRect(r);
+        }
+        else
+        {
+            boundary.addEllipse(r);
+        }
+    }
+    else if(areaLength > 0 && areaLength % 2 == 1 && type == QLatin1String("poly"))
+    {
+        QPolygon polygon;
+
+        for(int areaExpN = 1; areaExpN < areaLength; areaExpN += 2)
+        {
+            QPoint p(miniexp_to_int(miniexp_nth(areaExpN, areaExp)), miniexp_to_int(miniexp_nth(areaExpN + 1, areaExp)));
+
+            p.setY(size.height() - p.y());
+
+            polygon << p;
+        }
+
+        boundary.addPolygon(polygon);
+    }
+
+    return QTransform::fromScale(1.0 / size.width(), 1.0 / size.height()).map(boundary);
+}
+
+Link* loadLinkTarget(const QPainterPath& boundary, miniexp_t linkExp, int index, const QHash< QString, int >& indexByName)
+{
+    QString target;
+
+    miniexp_t targetExp = miniexp_nth(1, linkExp);
+
+    if(miniexp_stringp(targetExp))
+    {
+        target = QString::fromUtf8(miniexp_to_str(miniexp_nth(1, linkExp)));
+    }
+    else if(miniexp_length(targetExp) == 3 && qstrncmp(miniexp_to_name(miniexp_nth(0, targetExp)), "url", 3) == 0)
+    {
+        target = QString::fromUtf8(miniexp_to_str(miniexp_nth(1, targetExp)));
+    }
+
+    if(target.isEmpty())
+    {
+        return 0;
+    }
+
+    if(target.at(0) == QLatin1Char('#'))
+    {
+        target.remove(0, 1);
+
+        bool ok = false;
+        int targetPage = target.toInt(&ok);
+
+        if(!ok)
+        {
+            if(indexByName.contains(target))
+            {
+                targetPage = indexByName[target] + 1;
+            }
+            else
+            {
+                return 0;
+            }
+        }
+        else
+        {
+            targetPage = (target.at(0) == QLatin1Char('+') || target.at(0) == QLatin1Char('-')) ? index + targetPage : targetPage;
+        }
+
+        return new Link(boundary, targetPage);
+    }
+    else
+    {
+        return new Link(boundary, target);
+    }
+}
+
+QList< Link* > loadLinks(miniexp_t pageAnnoExp, const QSizeF& size, int index, const QHash< QString, int >& indexByName)
+{
+    QList< Link* > links;
+
+    const int pageAnnoLength = miniexp_length(pageAnnoExp);
+
+    for(int pageAnnoN = 0; pageAnnoN < pageAnnoLength; ++pageAnnoN)
+    {
+        miniexp_t linkExp = miniexp_nth(pageAnnoN, pageAnnoExp);
+
+        if(miniexp_length(linkExp) <= 3 || qstrncmp(miniexp_to_name(miniexp_nth(0, linkExp)), "maparea", 7 ) != 0 || !miniexp_symbolp(miniexp_nth(0, miniexp_nth(3, linkExp))))
+        {
+            continue;
+        }
+
+        const QString type = QString::fromUtf8(miniexp_to_name(miniexp_nth(0, miniexp_nth(3, linkExp))));
+
+        if(type == QLatin1String("rect") || type == QLatin1String("oval") || type == QLatin1String("poly"))
+        {
+            QPainterPath boundary = loadLinkBoundary(type, linkExp, size);
+
+            if(!boundary.isEmpty())
+            {
+                Link* link = loadLinkTarget(boundary, linkExp, index, indexByName);
+
+                if(link != 0)
+                {
+                    links.append(link);
+                }
+            }
+        }
+    }
+
+    return links;
+}
+
+QString loadText(miniexp_t textExp, const QRect& rect, int pageHeight)
+{
+    const int textLength = miniexp_length(textExp);
+
+    if(textLength >= 6 && miniexp_symbolp(miniexp_nth(0, textExp)))
+    {
+        const int xmin = miniexp_to_int(miniexp_nth(1, textExp));
+        const int ymin = miniexp_to_int(miniexp_nth(2, textExp));
+        const int xmax = miniexp_to_int(miniexp_nth(3, textExp));
+        const int ymax = miniexp_to_int(miniexp_nth(4, textExp));
+
+        if(rect.intersects(QRect(xmin, pageHeight - ymax, xmax - xmin, ymax - ymin)))
+        {
+            if(qstrncmp(miniexp_to_name(miniexp_nth(0, textExp)), "word", 4) == 0)
+            {
+                return QString::fromUtf8(miniexp_to_str(miniexp_nth(5, textExp)));
+            }
+            else
+            {
+                QStringList text;
+
+                for(int textN = 5; textN < textLength; ++textN)
+                {
+                    text.append(loadText(miniexp_nth(textN, textExp), rect, pageHeight));
+                }
+
+                if(qstrncmp(miniexp_to_name(miniexp_nth(0, textExp)), "line", 4) == 0)
+                {
+                    return text.join(" ");
+                }
+                else
+                {
+                    return text.join("\n");
+                }
+            }
+        }
+    }
+
+    return QString();
+}
+
+void loadOutline(miniexp_t outlineExp, int offset, QStandardItem* parent, const QHash< QString, int >& indexByName)
+{
+    const int outlineLength = miniexp_length(outlineExp);
+
+    for(int outlineN = qMax(0, offset); outlineN < outlineLength; ++outlineN)
+    {
+        miniexp_t bookmarkExp = miniexp_nth(outlineN, outlineExp);
+        const int bookmarkLength = miniexp_length(bookmarkExp);
+
+        if(bookmarkLength <= 1 || !miniexp_stringp(miniexp_nth(0, bookmarkExp)) || !miniexp_stringp(miniexp_nth(1, bookmarkExp)))
+        {
+            continue;
+        }
+
+        const QString title = QString::fromUtf8(miniexp_to_str(miniexp_nth(0, bookmarkExp)));
+        QString destination = QString::fromUtf8(miniexp_to_str(miniexp_nth(1, bookmarkExp)));
+
+        if(!title.isEmpty() && !destination.isEmpty())
+        {
+            if(destination.at(0) == QLatin1Char('#'))
+            {
+                destination.remove(0,1);
+
+                bool ok = false;
+                int destinationPage = destination.toInt(&ok);
+
+                if(!ok)
+                {
+                    if(indexByName.contains(destination))
+                    {
+                        destinationPage = indexByName[destination] + 1;
+                    }
+                    else
+                    {
+                        continue;
+                    }
+                }
+
+                QStandardItem* item = new QStandardItem(title);
+                item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
+
+                item->setData(destinationPage, Qt::UserRole + 1);
+
+                QStandardItem* pageItem = item->clone();
+                pageItem->setText(QString::number(destinationPage));
+                pageItem->setTextAlignment(Qt::AlignRight);
+
+                parent->appendRow(QList< QStandardItem* >() << item << pageItem);
+
+                if(bookmarkLength >= 3)
+                {
+                    loadOutline(bookmarkExp, 2, item, indexByName);
+                }
+            }
+        }
+    }
+}
+
+} // anonymous
+
+namespace qpdfview
+{
+
+namespace Model
+{
+
+DjVuPage::DjVuPage(const DjVuDocument* parent, int index, const ddjvu_pageinfo_t& pageinfo) :
     m_parent(parent),
     m_index(index),
     m_size(pageinfo.width, pageinfo.height),
@@ -84,16 +327,16 @@ Model::DjVuPage::DjVuPage(const DjVuDocument* parent, int index, const ddjvu_pag
 {
 }
 
-Model::DjVuPage::~DjVuPage()
+DjVuPage::~DjVuPage()
 {
 }
 
-QSizeF Model::DjVuPage::size() const
+QSizeF DjVuPage::size() const
 {
     return 72.0 / m_resolution * m_size;
 }
 
-QImage Model::DjVuPage::render(qreal horizontalResolution, qreal verticalResolution, Rotation rotation, const QRect& boundingRect) const
+QImage DjVuPage::render(qreal horizontalResolution, qreal verticalResolution, Rotation rotation, const QRect& boundingRect) const
 {
     QMutexLocker mutexLocker(&m_parent->m_mutex);
 
@@ -194,12 +437,10 @@ QImage Model::DjVuPage::render(qreal horizontalResolution, qreal verticalResolut
     return image;
 }
 
-QList< Model::Link* > Model::DjVuPage::links() const
+QList< Link* > DjVuPage::links() const
 {
     QMutexLocker mutexLocker(&m_parent->m_mutex);
 
-    QList< Link* > links;
-
     miniexp_t pageAnnoExp;
 
     while(true)
@@ -216,168 +457,14 @@ QList< Model::Link* > Model::DjVuPage::links() const
         }
     }
 
-    const int pageAnnoLength = miniexp_length(pageAnnoExp);
-
-    for(int pageAnnoN = 0; pageAnnoN < pageAnnoLength; ++pageAnnoN)
-    {
-        miniexp_t linkExp = miniexp_nth(pageAnnoN, pageAnnoExp);
-
-        if(miniexp_length(linkExp) <= 3 || qstrncmp(miniexp_to_name(miniexp_nth(0, linkExp)), "maparea", 7 ) != 0 || !miniexp_symbolp(miniexp_nth(0, miniexp_nth(3, linkExp))))
-        {
-            continue;
-        }
-
-        const QString type = QString::fromUtf8(miniexp_to_name(miniexp_nth(0, miniexp_nth(3, linkExp))));
-
-        if(type == QLatin1String("rect") || type == QLatin1String("oval") || type == QLatin1String("poly"))
-        {
-            // boundary
-
-            QPainterPath boundary;
-
-            miniexp_t areaExp = miniexp_nth(3, linkExp);
-            const int areaLength = miniexp_length( areaExp );
-
-            if(areaLength == 5 && (type == QLatin1String("rect") || type == QLatin1String("oval")))
-            {
-                QPoint p(miniexp_to_int(miniexp_nth(1, areaExp)), miniexp_to_int(miniexp_nth(2, areaExp)));
-                QSize s(miniexp_to_int(miniexp_nth(3, areaExp)), miniexp_to_int(miniexp_nth(4, areaExp)));
-
-                p.setY(m_size.height() - s.height() - p.y());
-
-                const QRectF r(p, s);
-
-                if(type == QLatin1String("rect"))
-                {
-                    boundary.addRect(r);
-                }
-                else
-                {
-                    boundary.addEllipse(r);
-                }
-            }
-            else if(areaLength > 0 && areaLength % 2 == 1 && type == QLatin1String("poly"))
-            {
-                QPolygon polygon;
-
-                for(int areaExpN = 1; areaExpN < areaLength; areaExpN += 2)
-                {
-                    QPoint p(miniexp_to_int(miniexp_nth(areaExpN, areaExp)), miniexp_to_int(miniexp_nth(areaExpN + 1, areaExp)));
-
-                    p.setY(m_size.height() - p.y());
-
-                    polygon << p;
-                }
-
-                boundary.addPolygon(polygon);
-            }
-
-            if(boundary.isEmpty())
-            {
-                continue;
-            }
-
-            boundary = QTransform::fromScale(1.0 / m_size.width(), 1.0 / m_size.height()).map(boundary);
-
-            // target
-
-            QString target;
-
-            miniexp_t targetExp = miniexp_nth(1, linkExp);
-
-            if(miniexp_stringp(targetExp))
-            {
-                target = QString::fromUtf8(miniexp_to_str(miniexp_nth(1, linkExp)));
-            }
-            else if(miniexp_length(targetExp) == 3 && qstrncmp(miniexp_to_name(miniexp_nth(0, targetExp)), "url", 3) == 0)
-            {
-                target = QString::fromUtf8(miniexp_to_str(miniexp_nth(1, targetExp)));
-            }
-
-            if(target.isEmpty())
-            {
-                continue;
-            }
-
-            if(target.at(0) == QLatin1Char('#'))
-            {
-                target.remove(0, 1);
-
-                bool ok = false;
-                int targetPage = target.toInt(&ok);
-
-                if(!ok)
-                {
-                    if(m_parent->m_indexByName.contains(target))
-                    {
-                        targetPage = m_parent->m_indexByName[target] + 1;
-                    }
-                    else
-                    {
-                        continue;
-                    }
-                }
-                else
-                {
-                    targetPage = (target.at(0) == QLatin1Char('+') || target.at(0) == QLatin1Char('-')) ? m_index + targetPage : targetPage;
-                }
-
-                links.append(new Link(boundary, targetPage));
-            }
-            else
-            {
-                links.append(new Link(boundary, target));
-            }
-        }
-    }
+    QList< Link* > links = loadLinks(pageAnnoExp, m_size, m_index, m_parent->m_indexByName);
 
     ddjvu_miniexp_release(m_parent->m_document, pageAnnoExp);
 
     return links;
 }
 
-static QString loadText(miniexp_t textExp, const QRect& rect, int pageHeight)
-{
-    const int textLength = miniexp_length(textExp);
-
-    if(textLength >= 6 && miniexp_symbolp(miniexp_nth(0, textExp)))
-    {
-        const int xmin = miniexp_to_int(miniexp_nth(1, textExp));
-        const int ymin = miniexp_to_int(miniexp_nth(2, textExp));
-        const int xmax = miniexp_to_int(miniexp_nth(3, textExp));
-        const int ymax = miniexp_to_int(miniexp_nth(4, textExp));
-
-        if(rect.intersects(QRect(xmin, pageHeight - ymax, xmax - xmin, ymax - ymin)))
-        {
-            if(qstrncmp(miniexp_to_name(miniexp_nth(0, textExp)), "word", 4) == 0)
-            {
-                return QString::fromUtf8(miniexp_to_str(miniexp_nth(5, textExp)));
-            }
-            else
-            {
-                QStringList text;
-
-                for(int textN = 5; textN < textLength; ++textN)
-                {
-                    text.append(loadText(miniexp_nth(textN, textExp), rect, pageHeight));
-                }
-
-                if(qstrncmp(miniexp_to_name(miniexp_nth(0, textExp)), "line", 4) == 0)
-                {
-                    return text.join(" ");
-                }
-                else
-                {
-                    return text.join("\n");
-                }
-            }
-        }
-    }
-
-    return QString();
-}
-
-QString Model::DjVuPage::text(const QRectF& rect) const
+QString DjVuPage::text(const QRectF& rect) const
 {
     QMutexLocker mutexLocker(&m_parent->m_mutex);
 
@@ -404,7 +491,7 @@ QString Model::DjVuPage::text(const QRectF& rect) const
     return text.trimmed();
 }
 
-QList< QRectF > Model::DjVuPage::search(const QString& text, bool matchCase) const
+QList< QRectF > DjVuPage::search(const QString& text, bool matchCase) const
 {
     QMutexLocker mutexLocker(&m_parent->m_mutex);
 
@@ -486,7 +573,7 @@ QList< QRectF > Model::DjVuPage::search(const QString& text, bool matchCase) con
 
     ddjvu_miniexp_release(m_parent->m_document, pageTextExp);
 
-    QTransform transform = QTransform::fromScale(72.0 / m_resolution, 72.0 / m_resolution);
+    const QTransform transform = QTransform::fromScale(72.0 / m_resolution, 72.0 / m_resolution);
 
     for(int index = 0; index < results.size(); ++index)
     {
@@ -496,7 +583,7 @@ QList< QRectF > Model::DjVuPage::search(const QString& text, bool matchCase) con
     return results;
 }
 
-Model::DjVuDocument::DjVuDocument(ddjvu_context_t* context, ddjvu_document_t* document) :
+DjVuDocument::DjVuDocument(ddjvu_context_t* context, ddjvu_document_t* document) :
     m_mutex(),
     m_context(context),
     m_document(document),
@@ -524,21 +611,21 @@ Model::DjVuDocument::DjVuDocument(ddjvu_context_t* context, ddjvu_document_t* do
     }
 }
 
-Model::DjVuDocument::~DjVuDocument()
+DjVuDocument::~DjVuDocument()
 {
     ddjvu_document_release(m_document);
     ddjvu_context_release(m_context);
     ddjvu_format_release(m_format);
 }
 
-int Model::DjVuDocument::numberOfPages() const
+int DjVuDocument::numberOfPages() const
 {
     QMutexLocker mutexLocker(&m_mutex);
 
     return ddjvu_document_get_pagenum(m_document);
 }
 
-Model::Page* Model::DjVuDocument::page(int index) const
+Page* DjVuDocument::page(int index) const
 {
     QMutexLocker mutexLocker(&m_mutex);
 
@@ -567,17 +654,17 @@ Model::Page* Model::DjVuDocument::page(int index) const
     return new DjVuPage(this, index, pageinfo);
 }
 
-QStringList Model::DjVuDocument::saveFilter() const
+QStringList DjVuDocument::saveFilter() const
 {
     return QStringList() << "DjVu (*.djvu *.djv)";
 }
 
-bool Model::DjVuDocument::canSave() const
+bool DjVuDocument::canSave() const
 {
     return true;
 }
 
-bool Model::DjVuDocument::save(const QString& filePath, bool withChanges) const
+bool DjVuDocument::save(const QString& filePath, bool withChanges) const
 {
     Q_UNUSED(withChanges);
 
@@ -602,65 +689,7 @@ bool Model::DjVuDocument::save(const QString& filePath, bool withChanges) const
     return !ddjvu_job_error(job);
 }
 
-static void loadOutline(miniexp_t outlineExp, int offset, QStandardItem* parent, const QHash< QString, int >& indexByName)
-{
-    const int outlineLength = miniexp_length(outlineExp);
-
-    for(int outlineN = qMax(0, offset); outlineN < outlineLength; ++outlineN)
-    {
-        miniexp_t bookmarkExp = miniexp_nth(outlineN, outlineExp);
-        const int bookmarkLength = miniexp_length(bookmarkExp);
-
-        if(bookmarkLength <= 1 || !miniexp_stringp(miniexp_nth(0, bookmarkExp)) || !miniexp_stringp(miniexp_nth(1, bookmarkExp)))
-        {
-            continue;
-        }
-
-        const QString title = QString::fromUtf8(miniexp_to_str(miniexp_nth(0, bookmarkExp)));
-        QString destination = QString::fromUtf8(miniexp_to_str(miniexp_nth(1, bookmarkExp)));
-
-        if(!title.isEmpty() && !destination.isEmpty())
-        {
-            if(destination.at(0) == QLatin1Char('#'))
-            {
-                destination.remove(0,1);
-
-                bool ok = false;
-                int destinationPage = destination.toInt(&ok);
-
-                if(!ok)
-                {
-                    if(indexByName.contains(destination))
-                    {
-                        destinationPage = indexByName[destination] + 1;
-                    }
-                    else
-                    {
-                        continue;
-                    }
-                }
-
-                QStandardItem* item = new QStandardItem(title);
-                item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
-
-                item->setData(destinationPage, Qt::UserRole + 1);
-
-                QStandardItem* pageItem = item->clone();
-                pageItem->setText(QString::number(destinationPage));
-                pageItem->setTextAlignment(Qt::AlignRight);
-
-                parent->appendRow(QList< QStandardItem* >() << item << pageItem);
-
-                if(bookmarkLength >= 3)
-                {
-                    loadOutline(bookmarkExp, 2, item, indexByName);
-                }
-            }
-        }
-    }
-}
-
-void Model::DjVuDocument::loadOutline(QStandardItemModel* outlineModel) const
+void DjVuDocument::loadOutline(QStandardItemModel* outlineModel) const
 {
     Document::loadOutline(outlineModel);
 
@@ -697,7 +726,7 @@ void Model::DjVuDocument::loadOutline(QStandardItemModel* outlineModel) const
     ddjvu_miniexp_release(m_document, outlineExp);
 }
 
-void Model::DjVuDocument::loadProperties(QStandardItemModel* propertiesModel) const
+void DjVuDocument::loadProperties(QStandardItemModel* propertiesModel) const
 {
     Document::loadProperties(propertiesModel);
 
@@ -755,6 +784,8 @@ void Model::DjVuDocument::loadProperties(QStandardItemModel* propertiesModel) co
     ddjvu_miniexp_release(m_document, annoExp);
 }
 
+} // Model
+
 DjVuPlugin::DjVuPlugin(QObject* parent) : QObject(parent)
 {
     setObjectName("DjVuPlugin");
@@ -785,8 +816,10 @@ Model::Document* DjVuPlugin::loadDocument(const QString& filePath) const
     return new Model::DjVuDocument(context, document);
 }
 
+} // qpdfview
+
 #if QT_VERSION < QT_VERSION_CHECK(5,0,0)
 
-Q_EXPORT_PLUGIN2(qpdfview_djvu, DjVuPlugin)
+Q_EXPORT_PLUGIN2(qpdfview_djvu, qpdfview::DjVuPlugin)
 
 #endif // QT_VERSION
