@@ -22,8 +22,8 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
-#ifndef PDFMODEL_H
-#define PDFMODEL_H
+#ifndef DJVUMODEL_H
+#define DJVUMODEL_H
 
 #include <QHash>
 #include <QMutex>
@@ -35,6 +35,9 @@ typedef struct ddjvu_pageinfo_s ddjvu_pageinfo_t;
 
 #include "model.h"
 
+namespace qpdfview
+{
+
 class DjVuPlugin;
 
 namespace Model
@@ -71,7 +74,7 @@ namespace Model
     class DjVuDocument : public Document
     {
         friend class DjVuPage;
-        friend class ::DjVuPlugin;
+        friend class qpdfview::DjVuPlugin;
 
     public:
         ~DjVuDocument();
@@ -106,7 +109,7 @@ namespace Model
 class DjVuPlugin : public QObject, Plugin
 {
     Q_OBJECT
-    Q_INTERFACES(Plugin)
+    Q_INTERFACES(qpdfview::Plugin)
 
 #if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
 
@@ -121,4 +124,6 @@ public:
 
 };
 
-#endif // PDFMODEL_H
+} // qpdfview
+
+#endif // DJVUMODEL_H
