@@ -1,6 +1,7 @@
 /*
 
-Copyright 2012-2013 Adam Reichold
+Copyright 2012-2014 Adam Reichold
+Copyright 2014 Dorian Scholz
 Copyright 2013 Thomas Etter
 
 This file is part of qpdfview.
@@ -33,6 +34,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <qmath.h>
 #include <QMenu>
 #include <QMessageBox>
+#include <QPrintEngine>
 #include <QProcess>
 #include <QProgressDialog>
 #include <QScrollBar>
@@ -62,6 +64,9 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include "miscellaneous.h"
 #include "documentlayout.h"
 
+namespace qpdfview
+{
+
 Settings* DocumentView::s_settings = 0;
 ShortcutHandler* DocumentView::s_shortcutHandler = 0;
 
@@ -71,9 +76,9 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     m_prefetchTimer(0),
     m_document(0),
     m_pages(),
-    m_filePath(),
-    m_currentPage(-1),
+    m_fileInfo(),
     m_wasModified(false),
+    m_currentPage(-1),
     m_past(),
     m_future(),
     m_layout(new SinglePageLayout),
@@ -146,7 +151,7 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
 
     connect(m_autoRefreshWatcher, SIGNAL(fileChanged(QString)), m_autoRefreshTimer, SLOT(start()));
 
-    connect(m_autoRefreshTimer, SIGNAL(timeout()), this, SLOT(refresh()));
+    connect(m_autoRefreshTimer, SIGNAL(timeout()), this, SLOT(on_autoRefresh_timeout()));
 
     // prefetch
 
@@ -161,6 +166,19 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     connect(this, SIGNAL(rotationChanged(Rotation)), m_prefetchTimer, SLOT(start()));
 
     connect(m_prefetchTimer, SIGNAL(timeout()), SLOT(on_prefetch_timeout()));
+
+    // settings
+
+    m_continuousMode = s_settings->documentView().continuousMode();
+    m_layout.reset(DocumentLayout::fromLayoutMode(s_settings->documentView().layoutMode()));
+    m_rightToLeftMode = s_settings->documentView().rightToLeftMode();
+
+    m_scaleMode = s_settings->documentView().scaleMode();
+    m_scaleFactor = s_settings->documentView().scaleFactor();
+    m_rotation = s_settings->documentView().rotation();
+
+    m_invertColors = s_settings->documentView().invertColors();
+    m_highlightAll = s_settings->documentView().highlightAll();
 }
 
 DocumentView::~DocumentView()
@@ -175,37 +193,17 @@ DocumentView::~DocumentView()
     delete m_document;
 }
 
-const QString& DocumentView::filePath() const
-{
-    return m_filePath;
-}
-
-int DocumentView::numberOfPages() const
-{
-    return m_pages.count();
-}
-
-int DocumentView::currentPage() const
-{
-    return m_currentPage;
-}
-
-bool DocumentView::wasModified() const
-{
-    return m_wasModified;
-}
-
 QStringList DocumentView::openFilter()
 {
     QStringList openFilter;
     QStringList supportedFormats;
 
-#ifdef WITH_PDF
+#if defined(WITH_PDF) || defined(WITH_FITZ)
 
     openFilter.append("Portable document format (*.pdf)");
     supportedFormats.append("*.pdf");
 
-#endif // WITH_PDF
+#endif // WITH_PDF // WITH_FITZ
 
 #ifdef WITH_PS
 
@@ -238,23 +236,20 @@ bool DocumentView::canSave() const
     return m_document->canSave();
 }
 
-bool DocumentView::continousMode() const
+void DocumentView::setContinuousMode(bool continuousMode)
 {
-    return m_continuousMode;
-}
-
-void DocumentView::setContinousMode(bool continousMode)
-{
-    if(m_continuousMode != continousMode)
+    if(m_continuousMode != continuousMode)
     {
-        m_continuousMode = continousMode;
+        m_continuousMode = continuousMode;
 
         qreal left = 0.0, top = 0.0;
         saveLeftAndTop(left, top);
 
         prepareView(left, top);
 
-        emit continousModeChanged(m_continuousMode);
+        emit continuousModeChanged(m_continuousMode);
+
+        s_settings->documentView().setContinuousMode(m_continuousMode);
     }
 }
 
@@ -267,14 +262,7 @@ void DocumentView::setLayoutMode(LayoutMode layoutMode)
 {
     if(m_layout->layoutMode() != layoutMode && layoutMode >= 0 && layoutMode < NumberOfLayoutModes)
     {
-        switch(layoutMode)
-        {
-        default:
-        case SinglePageMode: m_layout.reset(new SinglePageLayout); break;
-        case TwoPagesMode: m_layout.reset(new TwoPagesLayout); break;
-        case TwoPagesWithCoverPageMode: m_layout.reset(new TwoPagesWithCoverPageLayout); break;
-        case MultiplePagesMode: m_layout.reset(new MultiplePagesLayout); break;
-        }
+        m_layout.reset(DocumentLayout::fromLayoutMode(layoutMode));
 
         if(m_currentPage != m_layout->currentPage(m_currentPage))
         {
@@ -287,12 +275,24 @@ void DocumentView::setLayoutMode(LayoutMode layoutMode)
         prepareView();
 
         emit layoutModeChanged(layoutMode);
+
+        s_settings->documentView().setLayoutMode(layoutMode);
     }
 }
 
-ScaleMode DocumentView::scaleMode() const
+void DocumentView::setRightToLeftMode(bool rightToLeftMode)
 {
-    return m_scaleMode;
+    if(m_rightToLeftMode != rightToLeftMode)
+    {
+        m_rightToLeftMode = rightToLeftMode;
+
+        prepareScene();
+        prepareView();
+
+        emit rightToLeftModeChanged(m_rightToLeftMode);
+
+        s_settings->documentView().setRightToLeftMode(m_rightToLeftMode);
+    }
 }
 
 void DocumentView::setScaleMode(ScaleMode scaleMode)
@@ -308,12 +308,9 @@ void DocumentView::setScaleMode(ScaleMode scaleMode)
         prepareView(left, top);
 
         emit scaleModeChanged(m_scaleMode);
-    }
-}
 
-qreal DocumentView::scaleFactor() const
-{
-    return m_scaleFactor;
+        s_settings->documentView().setScaleMode(m_scaleMode);
+    }
 }
 
 void DocumentView::setScaleFactor(qreal scaleFactor)
@@ -332,12 +329,9 @@ void DocumentView::setScaleFactor(qreal scaleFactor)
         }
 
         emit scaleFactorChanged(m_scaleFactor);
-    }
-}
 
-Rotation DocumentView::rotation() const
-{
-    return m_rotation;
+        s_settings->documentView().setScaleFactor(m_scaleFactor);
+    }
 }
 
 void DocumentView::setRotation(Rotation rotation)
@@ -350,12 +344,9 @@ void DocumentView::setRotation(Rotation rotation)
         prepareView();
 
         emit rotationChanged(m_rotation);
-    }
-}
 
-bool DocumentView::invertColors() const
-{
-    return m_invertColors;
+        s_settings->documentView().setRotation(rotation);
+    }
 }
 
 void DocumentView::setInvertColors(bool invertColors)
@@ -377,12 +368,9 @@ void DocumentView::setInvertColors(bool invertColors)
         prepareBackground();
 
         emit invertColorsChanged(m_invertColors);
-    }
-}
 
-bool DocumentView::highlightAll() const
-{
-    return m_highlightAll;
+        s_settings->documentView().setInvertColors(m_invertColors);
+    }
 }
 
 void DocumentView::setHighlightAll(bool highlightAll)
@@ -402,12 +390,9 @@ void DocumentView::setHighlightAll(bool highlightAll)
         }
 
         emit highlightAllChanged(m_highlightAll);
-    }
-}
 
-RubberBandMode DocumentView::rubberBandMode() const
-{
-    return m_rubberBandMode;
+        s_settings->documentView().setHighlightAll(highlightAll);
+    }
 }
 
 void DocumentView::setRubberBandMode(RubberBandMode rubberBandMode)
@@ -435,11 +420,6 @@ int DocumentView::searchProgress() const
     return m_searchTask->progress();
 }
 
-Qt::Orientation DocumentView::thumbnailsOrientation() const
-{
-    return m_thumbnailsOrientation;
-}
-
 void DocumentView::setThumbnailsOrientation(Qt::Orientation thumbnailsOrientation)
 {
     if(m_thumbnailsOrientation != thumbnailsOrientation)
@@ -450,16 +430,6 @@ void DocumentView::setThumbnailsOrientation(Qt::Orientation thumbnailsOrientatio
     }
 }
 
-QStandardItemModel* DocumentView::outlineModel() const
-{
-    return m_outlineModel;
-}
-
-QStandardItemModel* DocumentView::propertiesModel() const
-{
-    return m_propertiesModel;
-}
-
 QStandardItemModel* DocumentView::fontsModel() const
 {
     QStandardItemModel* fontsModel = new QStandardItemModel();
@@ -469,16 +439,6 @@ QStandardItemModel* DocumentView::fontsModel() const
     return fontsModel;
 }
 
-const QVector< ThumbnailItem* >& DocumentView::thumbnailItems() const
-{
-    return m_thumbnailItems;
-}
-
-QGraphicsScene* DocumentView::thumbnailsScene() const
-{
-    return m_thumbnailsScene;
-}
-
 void DocumentView::show()
 {
     QGraphicsView::show();
@@ -488,31 +448,28 @@ void DocumentView::show()
 
 bool DocumentView::open(const QString& filePath)
 {
-    Model::Document* document = PluginHandler::loadDocument(filePath);
+    Model::Document* document = PluginHandler::instance()->loadDocument(filePath);
 
     if(document != 0)
     {
-        if(document->isLocked())
+        if(!checkDocument(filePath, document))
         {
-            QString password = QInputDialog::getText(this, tr("Unlock %1").arg(QFileInfo(filePath).completeBaseName()), tr("Password:"), QLineEdit::Password);
-
-            if(document->unlock(password))
-            {
-                delete document;
-                return false;
-            }
+            delete document;
+            return false;
         }
 
-        m_filePath = filePath;
-        m_currentPage = 1;
-
+        m_fileInfo.setFile(filePath);
         m_wasModified = false;
 
+        m_currentPage = 1;
+
         m_past.clear();
         m_future.clear();
 
         prepareDocument(document);
 
+        loadDocumentDefaults();
+
         prepareScene();
         prepareView();
 
@@ -520,7 +477,6 @@ bool DocumentView::open(const QString& filePath)
 
         emit documentChanged();
 
-        emit filePathChanged(m_filePath);
         emit numberOfPagesChanged(m_pages.count());
         emit currentPageChanged(m_currentPage);
 
@@ -532,28 +488,23 @@ bool DocumentView::open(const QString& filePath)
 
 bool DocumentView::refresh()
 {
-    Model::Document* document = PluginHandler::loadDocument(m_filePath);
+    Model::Document* document = PluginHandler::instance()->loadDocument(m_fileInfo.filePath());
 
     if(document != 0)
     {
-        if(document->isLocked())
+        if(!checkDocument(m_fileInfo.filePath(), document))
         {
-            QString password = QInputDialog::getText(this, tr("Unlock %1").arg(QFileInfo(m_filePath).completeBaseName()), tr("Password:"), QLineEdit::Password);
-
-            if(document->unlock(password))
-            {
-                delete document;
-                return false;
-            }
+            delete document;
+            return false;
         }
 
         qreal left = 0.0, top = 0.0;
         saveLeftAndTop(left, top);
 
-        m_currentPage = qMin(m_currentPage, document->numberOfPages());
-
         m_wasModified = false;
 
+        m_currentPage = qMin(m_currentPage, document->numberOfPages());
+
         prepareDocument(document);
 
         prepareScene();
@@ -672,7 +623,7 @@ void DocumentView::jumpToPage(int page, bool trackChange, qreal changeLeft, qrea
 
             m_currentPage = m_layout->currentPage(page);
 
-            prepareView(changeLeft, changeTop);
+            prepareView(changeLeft, changeTop, page);
 
             emit currentPageChanged(m_currentPage, trackChange);
         }
@@ -915,15 +866,14 @@ void DocumentView::startPresentation()
     presentationView->setAttribute(Qt::WA_DeleteOnClose);
 
     connect(this, SIGNAL(destroyed()), presentationView, SLOT(close()));
-    connect(this, SIGNAL(filePathChanged(QString)), presentationView, SLOT(close()));
-    connect(this, SIGNAL(numberOfPagesChanged(int)), presentationView, SLOT(close()));
+    connect(this, SIGNAL(documentChanged()), presentationView, SLOT(close()));
 
     presentationView->setRotation(rotation());
     presentationView->setInvertColors(invertColors());
 
     presentationView->jumpToPage(currentPage(), false);
 
-    if(s_settings->presentationView().sync())
+    if(s_settings->presentationView().synchronize())
     {
         connect(this, SIGNAL(currentPageChanged(int,bool)), presentationView, SLOT(jumpToPage(int,bool)));
         connect(presentationView, SIGNAL(currentPageChanged(int,bool)), this, SLOT(jumpToPage(int,bool)));
@@ -932,42 +882,58 @@ void DocumentView::startPresentation()
 
 void DocumentView::on_verticalScrollBar_valueChanged(int value)
 {
-    if(m_continuousMode)
+    if(!m_continuousMode)
     {
-        const QRectF visibleRect = mapToScene(viewport()->rect()).boundingRect();
+        return;
+    }
 
-        foreach(PageItem* page, m_pageItems)
+    const QRectF visibleRect = mapToScene(viewport()->rect()).boundingRect();
+
+    foreach(PageItem* page, m_pageItems)
+    {
+        if(!page->boundingRect().translated(page->pos()).intersects(visibleRect))
         {
-            if(!page->boundingRect().translated(page->pos()).intersects(visibleRect))
-            {
-                page->cancelRender();
-            }
+            page->cancelRender();
         }
+    }
 
-        const QMap< qreal, int >::iterator lowerBound = m_heightToIndex.lowerBound(-value);
+    const QMap< qreal, int >::iterator lowerBound = m_heightToIndex.lowerBound(-value);
 
-        if(lowerBound != m_heightToIndex.end())
+    if(lowerBound != m_heightToIndex.end())
+    {
+        const int page = lowerBound.value() + 1;
+
+        if(m_currentPage != page)
         {
-            const int page = lowerBound.value() + 1;
+            m_currentPage = page;
 
-            if(m_currentPage != page)
+            emit currentPageChanged(m_currentPage);
+
+            if(s_settings->documentView().highlightCurrentThumbnail())
             {
-                m_currentPage = page;
-
-                emit currentPageChanged(m_currentPage);
-
-                if(s_settings->documentView().highlightCurrentThumbnail())
+                for(int index = 0; index < m_thumbnailItems.count(); ++index)
                 {
-                    for(int index = 0; index < m_thumbnailItems.count(); ++index)
-                    {
-                        m_thumbnailItems.at(index)->setCurrent(index == m_currentPage - 1);
-                    }
+                    m_thumbnailItems.at(index)->setCurrent(index == m_currentPage - 1);
                 }
             }
         }
     }
 }
 
+void DocumentView::on_autoRefresh_timeout()
+{
+    if(m_fileInfo.exists())
+    {
+        refresh();
+    }
+    else
+    {
+        m_wasModified = true;
+
+        emit documentModified();
+    }
+}
+
 void DocumentView::on_prefetch_timeout()
 {
     const QPair< int, int > prefetchRange = m_layout->prefetchRange(m_currentPage, m_pages.count());
@@ -1012,7 +978,7 @@ void DocumentView::on_searchTask_resultsReady(int index, QList< QRectF > results
     }
 }
 
-void DocumentView::on_pages_linkClicked(int page, qreal left, qreal top)
+void DocumentView::on_pages_linkClicked(bool newTab, int page, qreal left, qreal top)
 {
     page = qMax(page, 1);
     page = qMin(page, m_pages.count());
@@ -1023,7 +989,14 @@ void DocumentView::on_pages_linkClicked(int page, qreal left, qreal top)
     top = top >= 0.0 ? top : 0.0;
     top = top <= 1.0 ? top : 1.0;
 
-    jumpToPage(page, true, left, top);
+    if(newTab)
+    {
+        emit linkClicked(page);
+    }
+    else
+    {
+        jumpToPage(page, true, left, top);
+    }
 }
 
 void DocumentView::on_pages_linkClicked(const QString& url)
@@ -1034,7 +1007,7 @@ void DocumentView::on_pages_linkClicked(const QString& url)
 
         if(resolvedUrl.isRelative() && QFileInfo(url).isRelative())
         {
-            resolvedUrl.setPath(QFileInfo(m_filePath).dir().filePath(url));
+            resolvedUrl.setPath(m_fileInfo.dir().filePath(url));
         }
 
         QDesktopServices::openUrl(resolvedUrl);
@@ -1047,7 +1020,7 @@ void DocumentView::on_pages_linkClicked(const QString& url)
 
 void DocumentView::on_pages_linkClicked(const QString& fileName, int page)
 {
-    const QString filePath = QFileInfo(fileName).isAbsolute() ? fileName : QFileInfo(m_filePath).dir().filePath(fileName);
+    const QString filePath = QFileInfo(fileName).isAbsolute() ? fileName : m_fileInfo.dir().filePath(fileName);
 
     emit linkClicked(filePath, page);
 }
@@ -1057,7 +1030,7 @@ void DocumentView::on_pages_rubberBandFinished()
     setRubberBandMode(ModifiersMode);
 }
 
-void DocumentView::on_pages_sourceRequested(int page, const QPointF& pos)
+void DocumentView::on_pages_editSourceRequested(int page, const QPointF& pos)
 {
 #ifdef WITH_SYNCTEX
 
@@ -1066,9 +1039,7 @@ void DocumentView::on_pages_sourceRequested(int page, const QPointF& pos)
         return;
     }
 
-    const QFileInfo fileInfo(m_filePath);
-
-    synctex_scanner_t scanner = synctex_scanner_new_with_output_file(fileInfo.absoluteFilePath().toLocal8Bit(), 0, 1);
+    synctex_scanner_t scanner = synctex_scanner_new_with_output_file(m_fileInfo.absoluteFilePath().toLocal8Bit(), 0, 1);
 
     if(scanner != 0)
     {
@@ -1083,7 +1054,7 @@ void DocumentView::on_pages_sourceRequested(int page, const QPointF& pos)
                 sourceLine = sourceLine >= 0 ? sourceLine : 0;
                 sourceColumn = sourceColumn >= 0 ? sourceColumn : 0;
 
-                QProcess::startDetached(s_settings->documentView().sourceEditor().arg(fileInfo.dir().absoluteFilePath(sourceName), QString::number(sourceLine), QString::number(sourceColumn)));
+                QProcess::startDetached(s_settings->documentView().sourceEditor().arg(m_fileInfo.dir().absoluteFilePath(sourceName), QString::number(sourceLine), QString::number(sourceColumn)));
 
                 break;
             }
@@ -1093,7 +1064,7 @@ void DocumentView::on_pages_sourceRequested(int page, const QPointF& pos)
     }
     else
     {
-        QMessageBox::warning(this, tr("Warning"), tr("SyncTeX data for '%1' could not be found.").arg(fileInfo.absoluteFilePath()));
+        QMessageBox::warning(this, tr("Warning"), tr("SyncTeX data for '%1' could not be found.").arg(m_fileInfo.absoluteFilePath()));
     }
 
 #else
@@ -1104,9 +1075,26 @@ void DocumentView::on_pages_sourceRequested(int page, const QPointF& pos)
 #endif // WITH_SYNCTEX
 }
 
+void DocumentView::on_pages_zoomToSelectionRequested(int page, const QRectF& rect)
+{
+    const qreal visibleWidth = m_layout->visibleWidth(viewport()->width());
+    const qreal visibleHeight = m_layout->visibleHeight(viewport()->height());
+
+    const qreal unscaledWidth = m_pageItems.at(page - 1)->unscaledWidth();
+    const qreal unscaledHeight = m_pageItems.at(page - 1)->unscaledHeight();
+
+    setScaleFactor(qMin(qMin(visibleWidth / unscaledWidth / rect.width(),
+                             visibleHeight / unscaledHeight / rect.height()),
+                        Defaults::DocumentView::maximumScaleFactor()));
+
+    jumpToPage(page, false, rect.left(), rect.top());
+}
+
 void DocumentView::on_pages_wasModified()
 {
     m_wasModified = true;
+
+    emit documentModified();
 }
 
 void DocumentView::resizeEvent(QResizeEvent* event)
@@ -1161,19 +1149,11 @@ void DocumentView::keyPressEvent(QKeyEvent* event)
         return;
     }
 
-    if(maskedKey != -1)
-    {
-        QKeyEvent keyEvent(event->type(), maskedKey, Qt::NoModifier, event->text(), event->isAutoRepeat(), event->count());
-        QGraphicsView::keyPressEvent(&keyEvent);
-    }
-    else
+    if(maskedKey == -1)
     {
         QGraphicsView::keyPressEvent(event);
     }
-
-    if(maskedKey == Qt::Key_PageUp || maskedKey == Qt::Key_PageDown ||
-       maskedKey == Qt::Key_Up || maskedKey == Qt::Key_Down ||
-       maskedKey == Qt::Key_Left || maskedKey == Qt::Key_Right)
+    else
     {
         foreach(const PageItem* page, m_pageItems)
         {
@@ -1221,6 +1201,9 @@ void DocumentView::keyPressEvent(QKeyEvent* event)
             event->accept();
             return;
         }
+
+        QKeyEvent keyEvent(event->type(), maskedKey, Qt::NoModifier, event->text(), event->isAutoRepeat(), event->count());
+        QGraphicsView::keyPressEvent(&keyEvent);
     }
 }
 
@@ -1328,6 +1311,13 @@ bool DocumentView::printUsingCUPS(QPrinter* printer, const PrintOptions& printOp
             num_options = cupsAddOption(dest->options[index].name, dest->options[index].value, num_options, &options);
         }
 
+        QStringList cupsOptions = printer->printEngine()->property(QPrintEngine::PrintEnginePropertyKey(0xfe00)).toStringList();
+
+        for(int index = 0; index < cupsOptions.count() - 1; index += 2)
+        {
+            num_options = cupsAddOption(cupsOptions.at(index).toLocal8Bit(), cupsOptions.at(index + 1).toLocal8Bit(), num_options, &options);
+        }
+
 #if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
 
         num_options = cupsAddOption("copies", QString::number(printer->copyCount()).toLocal8Bit(), num_options, &options);
@@ -1472,7 +1462,7 @@ bool DocumentView::printUsingCUPS(QPrinter* printer, const PrintOptions& printOp
 
             if(m_document->save(temporaryFile.fileName(), true))
             {
-                jobId = cupsPrintFile(dest->name, QFileInfo(temporaryFile).absoluteFilePath().toLocal8Bit(), QFileInfo(m_filePath).completeBaseName().toLocal8Bit(), num_options, options);
+                jobId = cupsPrintFile(dest->name, temporaryFile.fileName().toLocal8Bit(), m_fileInfo.completeBaseName().toLocal8Bit(), num_options, options);
 
                 if(jobId < 1)
                 {
@@ -1497,7 +1487,7 @@ bool DocumentView::printUsingCUPS(QPrinter* printer, const PrintOptions& printOp
 bool DocumentView::printUsingQt(QPrinter* printer, const PrintOptions& printOptions, int fromPage, int toPage)
 {
     QScopedPointer< QProgressDialog > progressDialog(new QProgressDialog(this));
-    progressDialog->setLabelText(tr("Printing '%1'...").arg(m_filePath));
+    progressDialog->setLabelText(tr("Printing '%1'...").arg(m_fileInfo.completeBaseName()));
     progressDialog->setRange(fromPage - 1, toPage);
 
     QPainter painter(printer);
@@ -1565,13 +1555,30 @@ void DocumentView::saveLeftAndTop(qreal& left, qreal& top) const
     top = top >= 0.0 ? top : 0.0;
 }
 
-void DocumentView::loadFallbackOutline()
+bool DocumentView::checkDocument(const QString& filePath, Model::Document* document)
 {
-    if(m_outlineModel->rowCount() > 0)
+    if(document->isLocked())
     {
-        return;
+        QString password = QInputDialog::getText(this, tr("Unlock %1").arg(QFileInfo(filePath).completeBaseName()), tr("Password:"), QLineEdit::Password);
+
+        if(document->unlock(password))
+        {
+            return false;
+        }
     }
 
+    if(document->numberOfPages() == 0)
+    {
+        qWarning() << "No pages were found in document at" << filePath;
+
+        return false;
+    }
+
+    return true;
+}
+
+void DocumentView::loadFallbackOutline()
+{
     for(int page = 1; page <= m_pages.count(); ++page)
     {
         QStandardItem* item = new QStandardItem(tr("Page %1").arg(page));
@@ -1587,6 +1594,32 @@ void DocumentView::loadFallbackOutline()
     }
 }
 
+void DocumentView::loadDocumentDefaults()
+{
+    if(m_document->wantsContinuousMode())
+    {
+        m_continuousMode = true;
+    }
+
+    if(m_document->wantsSinglePageMode())
+    {
+        m_layout.reset(new SinglePageLayout);
+    }
+    else if(m_document->wantsTwoPagesMode())
+    {
+        m_layout.reset(new TwoPagesLayout);
+    }
+    else if(m_document->wantsTwoPagesWithCoverPageMode())
+    {
+        m_layout.reset(new TwoPagesWithCoverPageLayout);
+    }
+
+    if(m_document->wantsRightToLeftMode())
+    {
+        m_rightToLeftMode = true;
+    }
+}
+
 void DocumentView::prepareDocument(Model::Document* document)
 {
     m_prefetchTimer->blockSignals(true);
@@ -1599,32 +1632,31 @@ void DocumentView::prepareDocument(Model::Document* document)
 
     qDeleteAll(m_pages);
 
-    if(m_document != 0)
-    {
-        delete m_document;
-
-        if(!m_autoRefreshWatcher->files().isEmpty())
-        {
-            m_autoRefreshWatcher->removePaths(m_autoRefreshWatcher->files());
-        }
-    }
-
+    delete m_document;
     m_document = document;
 
-    if(s_settings->documentView().autoRefresh())
-    {
-        m_autoRefreshWatcher->addPath(m_filePath);
-    }
-
-    m_document->setPaperColor(s_settings->pageItem().paperColor());
+    const int numberOfPages = m_document->numberOfPages();
 
     m_pages.clear();
+    m_pages.reserve(numberOfPages);
 
-    for(int index = 0; index < m_document->numberOfPages(); ++index)
+    for(int index = 0; index < numberOfPages; ++index)
     {
         m_pages.append(m_document->page(index));
     }
 
+    if(!m_autoRefreshWatcher->files().isEmpty())
+    {
+        m_autoRefreshWatcher->removePaths(m_autoRefreshWatcher->files());
+    }
+
+    if(s_settings->documentView().autoRefresh())
+    {
+        m_autoRefreshWatcher->addPath(m_fileInfo.filePath());
+    }
+
+    m_document->setPaperColor(s_settings->pageItem().paperColor());
+
     preparePages();
     prepareThumbnails();
     prepareBackground();
@@ -1632,7 +1664,10 @@ void DocumentView::prepareDocument(Model::Document* document)
     m_document->loadOutline(m_outlineModel);
     m_document->loadProperties(m_propertiesModel);
 
-    loadFallbackOutline();
+    if(m_outlineModel->rowCount() == 0)
+    {
+        loadFallbackOutline();
+    }
 
     if(s_settings->documentView().prefetch())
     {
@@ -1656,13 +1691,14 @@ void DocumentView::preparePages()
         scene()->addItem(page);
         m_pageItems.append(page);
 
-        connect(page, SIGNAL(linkClicked(int,qreal,qreal)), SLOT(on_pages_linkClicked(int,qreal,qreal)));
+        connect(page, SIGNAL(linkClicked(bool,int,qreal,qreal)), SLOT(on_pages_linkClicked(bool,int,qreal,qreal)));
         connect(page, SIGNAL(linkClicked(QString)), SLOT(on_pages_linkClicked(QString)));
         connect(page, SIGNAL(linkClicked(QString,int)), SLOT(on_pages_linkClicked(QString,int)));
 
         connect(page, SIGNAL(rubberBandFinished()), SLOT(on_pages_rubberBandFinished()));
 
-        connect(page, SIGNAL(sourceRequested(int,QPointF)), SLOT(on_pages_sourceRequested(int,QPointF)));
+        connect(page, SIGNAL(editSourceRequested(int,QPointF)), SLOT(on_pages_editSourceRequested(int,QPointF)));
+        connect(page, SIGNAL(zoomToSelectionRequested(int,QRectF)), SLOT(on_pages_zoomToSelectionRequested(int,QRectF)));
 
         connect(page, SIGNAL(wasModified()), SLOT(on_pages_wasModified()));
     }
@@ -1682,7 +1718,7 @@ void DocumentView::prepareThumbnails()
         m_thumbnailsScene->addItem(page);
         m_thumbnailItems.append(page);
 
-        connect(page, SIGNAL(linkClicked(int,qreal,qreal)), SLOT(on_pages_linkClicked(int,qreal,qreal)));
+        connect(page, SIGNAL(linkClicked(bool,int,qreal,qreal)), SLOT(on_pages_linkClicked(bool,int,qreal,qreal)));
     }
 }
 
@@ -1717,7 +1753,7 @@ void DocumentView::prepareScene()
 
     foreach(PageItem* page, m_pageItems)
     {
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+#if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
 
         page->setDevicePixelRatio(devicePixelRatio());
 
@@ -1725,23 +1761,10 @@ void DocumentView::prepareScene()
 
         page->setResolution(logicalDpiX(), logicalDpiY());
 
-        qreal pageWidth = 0.0;
-        qreal pageHeight = 0.0;
+        page->setRotation(m_rotation);
 
-        switch(m_rotation)
-        {
-        default:
-        case RotateBy0:
-        case RotateBy180:
-            pageWidth = logicalDpiX() / 72.0 * page->size().width();
-            pageHeight = logicalDpiY() / 72.0 * page->size().height();
-            break;
-        case RotateBy90:
-        case RotateBy270:
-            pageWidth = logicalDpiX() / 72.0 * page->size().height();
-            pageHeight = logicalDpiY() / 72.0 * page->size().width();
-            break;
-        }
+        const qreal unscaledWidth = page->unscaledWidth();
+        const qreal unscaledHeight = page->unscaledHeight();
 
         switch(m_scaleMode)
         {
@@ -1750,34 +1773,29 @@ void DocumentView::prepareScene()
             page->setScaleFactor(m_scaleFactor);
             break;
         case FitToPageWidthMode:
-            page->setScaleFactor(visibleWidth / pageWidth);
+            page->setScaleFactor(visibleWidth / unscaledWidth);
             break;
         case FitToPageSizeMode:
-            page->setScaleFactor(qMin(visibleWidth / pageWidth, visibleHeight / pageHeight));
+            page->setScaleFactor(qMin(visibleWidth / unscaledWidth, visibleHeight / unscaledHeight));
             break;
         }
-
-        page->setRotation(m_rotation);
     }
 
     // prepare layout
 
     m_heightToIndex.clear();
 
-    qreal pageHeight = 0.0;
-
     qreal left = 0.0;
     qreal right = 0.0;
     qreal height = s_settings->documentView().pageSpacing();
 
-    m_layout->prepareLayout(m_pageItems,
-                            m_heightToIndex, pageHeight,
-                            left, right, height);
+    m_layout->prepareLayout(m_pageItems, m_rightToLeftMode,
+                            m_heightToIndex, left, right, height);
 
     scene()->setSceneRect(left, 0.0, right - left, height);
 }
 
-void DocumentView::prepareView(qreal changeLeft, qreal changeTop)
+void DocumentView::prepareView(qreal changeLeft, qreal changeTop, int visiblePage)
 {
     const bool highlightCurrentThumbnail = s_settings->documentView().highlightCurrentThumbnail();
 
@@ -1789,6 +1807,8 @@ void DocumentView::prepareView(qreal changeLeft, qreal changeTop)
     int horizontalValue = 0;
     int verticalValue = 0;
 
+    visiblePage = visiblePage == 0 ? m_currentPage : visiblePage;
+
     for(int index = 0; index < m_pageItems.count(); ++index)
     {
         PageItem* page = m_pageItems.at(index);
@@ -1797,12 +1817,6 @@ void DocumentView::prepareView(qreal changeLeft, qreal changeTop)
         if(m_continuousMode)
         {
             page->setVisible(true);
-
-            if(index == m_currentPage - 1)
-            {
-                horizontalValue = qFloor(boundingRect.left() + changeLeft * boundingRect.width());
-                verticalValue = qFloor(boundingRect.top() + changeTop * boundingRect.height());
-            }
         }
         else
         {
@@ -1812,12 +1826,6 @@ void DocumentView::prepareView(qreal changeLeft, qreal changeTop)
 
                 top = boundingRect.top() - s_settings->documentView().pageSpacing();
                 height = boundingRect.height() + 2.0 * s_settings->documentView().pageSpacing();
-
-                if(index == m_currentPage - 1)
-                {
-                    horizontalValue = qFloor(boundingRect.left() + changeLeft * boundingRect.width());
-                    verticalValue = qFloor(boundingRect.top() + changeTop * boundingRect.height());
-                }
             }
             else
             {
@@ -1827,15 +1835,18 @@ void DocumentView::prepareView(qreal changeLeft, qreal changeTop)
             }
         }
 
-        if(m_currentResult != m_results.end())
+        if(index == visiblePage - 1)
         {
-            if(m_currentResult.key() == index)
-            {
-                m_highlight->setPos(page->pos());
-                m_highlight->setTransform(page->transform());
+            horizontalValue = qFloor(boundingRect.left() + changeLeft * boundingRect.width());
+            verticalValue = qFloor(boundingRect.top() + changeTop * boundingRect.height());
+        }
 
-                page->stackBefore(m_highlight);
-            }
+        if(m_currentResult != m_results.end() && m_currentResult.key() == index)
+        {
+            m_highlight->setPos(page->pos());
+            m_highlight->setTransform(page->transform());
+
+            page->stackBefore(m_highlight);
         }
 
         m_thumbnailItems.at(index)->setCurrent(highlightCurrentThumbnail && (index == m_currentPage - 1));
@@ -1877,7 +1888,7 @@ void DocumentView::prepareThumbnailsScene()
 
         // prepare scale factor
 
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+#if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
 
         page->setDevicePixelRatio(devicePixelRatio());
 
@@ -1885,10 +1896,7 @@ void DocumentView::prepareThumbnailsScene()
 
         page->setResolution(logicalDpiX(), logicalDpiY());
 
-        const qreal pageWidth = logicalDpiX() / 72.0 * page->size().width();
-        const qreal pageHeight = logicalDpiY() / 72.0 * page->size().height();
-
-        page->setScaleFactor(qMin(s_settings->documentView().thumbnailSize() / pageWidth, s_settings->documentView().thumbnailSize() / pageHeight));
+        page->setScaleFactor(qMin(s_settings->documentView().thumbnailSize() / page->unscaledWidth(), s_settings->documentView().thumbnailSize() / page->unscaledHeight()));
 
         // prepare layout
 
@@ -1940,3 +1948,5 @@ DocumentView::Results::iterator DocumentView::previousResult(const Results::iter
 {
     return result != m_results.begin() ? result - 1 : m_results.end();
 }
+
+} // qpdfview
