@@ -1,6 +1,7 @@
 /*
 
-Copyright 2012-2013 Adam Reichold
+Copyright 2012-2014 Adam Reichold
+Copyright 2014 Dorian Scholz
 
 This file is part of qpdfview.
 
@@ -22,6 +23,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef DOCUMENTVIEW_H
 #define DOCUMENTVIEW_H
 
+#include <QFileInfo>
 #include <QGraphicsView>
 #include <QMap>
 
@@ -33,6 +35,9 @@ class QStandardItemModel;
 #include "global.h"
 #include "printoptions.h"
 
+namespace qpdfview
+{
+
 namespace Model
 {
 class Annotation;
@@ -56,70 +61,75 @@ public:
     explicit DocumentView(QWidget* parent = 0);
     ~DocumentView();
 
-    const QString& filePath() const;
-    int numberOfPages() const;
-    int currentPage() const;
+    inline const QFileInfo& fileInfo() const { return m_fileInfo; }
+    inline bool wasModified() const { return m_wasModified; }
 
-    bool wasModified() const;
+    inline int numberOfPages() const { return m_pages.count(); }
+    inline int currentPage() const { return m_currentPage; }
 
     static QStringList openFilter();
     QStringList saveFilter() const;
 
     bool canSave() const;
 
-    bool continousMode() const;
-    void setContinousMode(bool continousMode);
+    inline bool continuousMode() const { return m_continuousMode; }
+    void setContinuousMode(bool continuousMode);
 
     LayoutMode layoutMode() const;
     void setLayoutMode(LayoutMode layoutMode);
 
-    ScaleMode scaleMode() const;
+    inline bool rightToLeftMode() const { return m_rightToLeftMode; }
+    void setRightToLeftMode(bool rightToLeftMode);
+
+    inline ScaleMode scaleMode() const { return m_scaleMode; }
     void setScaleMode(ScaleMode scaleMode);
 
-    qreal scaleFactor() const;
+    inline qreal scaleFactor() const { return m_scaleFactor; }
     void setScaleFactor(qreal scaleFactor);
 
-    Rotation rotation() const;
+    inline Rotation rotation() const { return m_rotation; }
     void setRotation(Rotation rotation);
 
-    bool invertColors() const;
+    inline bool invertColors() const { return m_invertColors; }
     void setInvertColors(bool invertColors);
 
-    bool highlightAll() const;
+    inline bool highlightAll() const { return m_highlightAll; }
     void setHighlightAll(bool highlightAll);
 
-    RubberBandMode rubberBandMode() const;
+    inline RubberBandMode rubberBandMode() const { return m_rubberBandMode; }
     void setRubberBandMode(RubberBandMode rubberBandMode);
 
     bool searchWasCanceled() const;
     int searchProgress() const;
 
-    Qt::Orientation thumbnailsOrientation() const;
+    inline Qt::Orientation thumbnailsOrientation() const { return m_thumbnailsOrientation; }
     void setThumbnailsOrientation(Qt::Orientation thumbnailsOrientation);
 
-    const QVector< ThumbnailItem* >& thumbnailItems() const;
-    QGraphicsScene* thumbnailsScene() const;
+    inline const QVector< ThumbnailItem* >& thumbnailItems() const { return m_thumbnailItems; }
+    inline QGraphicsScene* thumbnailsScene() const { return m_thumbnailsScene; }
 
-    QStandardItemModel* outlineModel() const;
-    QStandardItemModel* propertiesModel() const;
+    inline QStandardItemModel* outlineModel() const { return m_outlineModel; }
+    inline QStandardItemModel* propertiesModel() const { return m_propertiesModel; }
 
     QStandardItemModel* fontsModel() const;
 
 signals:
     void documentChanged();
+    void documentModified();
 
-    void filePathChanged(const QString& filePath);
     void numberOfPagesChanged(int numberOfPages);
     void currentPageChanged(int currentPage, bool trackChange = false);
 
     void canJumpChanged(bool backward, bool forward);
 
-    void continousModeChanged(bool continousMode);
+    void continuousModeChanged(bool continuousMode);
     void layoutModeChanged(LayoutMode layoutMode);
+    void rightToLeftModeChanged(bool rightToLeftMode);
     void scaleModeChanged(ScaleMode scaleMode);
     void scaleFactorChanged(qreal scaleFactor);
     void rotationChanged(Rotation rotation);
 
+    void linkClicked(int page);
     void linkClicked(const QString& filePath, int page);
 
     void invertColorsChanged(bool invertColors);
@@ -170,19 +180,21 @@ public slots:
 protected slots:
     void on_verticalScrollBar_valueChanged(int value);
 
+    void on_autoRefresh_timeout();
     void on_prefetch_timeout();
 
     void on_temporaryHighlight_timeout();
 
     void on_searchTask_resultsReady(int index, QList< QRectF > results);
 
-    void on_pages_linkClicked(int page, qreal left, qreal top);
+    void on_pages_linkClicked(bool newTab, int page, qreal left, qreal top);
     void on_pages_linkClicked(const QString& url);
     void on_pages_linkClicked(const QString& fileName, int page);
 
     void on_pages_rubberBandFinished();
 
-    void on_pages_sourceRequested(int page, const QPointF& pos);
+    void on_pages_editSourceRequested(int page, const QPointF& pos);
+    void on_pages_zoomToSelectionRequested(int page, const QRectF& rect);
 
     void on_pages_wasModified();
 
@@ -206,13 +218,13 @@ private:
     QTimer* m_prefetchTimer;
 
     Model::Document* m_document;
-    QList< Model::Page* > m_pages;
-
-    QString m_filePath;
-    int m_currentPage;
+    QVector< Model::Page* > m_pages;
 
+    QFileInfo m_fileInfo;
     bool m_wasModified;
 
+    int m_currentPage;
+
 #ifdef WITH_CUPS
 
     bool printUsingCUPS(QPrinter* printer, const PrintOptions& printOptions, int fromPage, int toPage);
@@ -239,6 +251,7 @@ private:
     QScopedPointer< DocumentLayout > m_layout;
 
     bool m_continuousMode;
+    bool m_rightToLeftMode;
     ScaleMode m_scaleMode;
     qreal m_scaleFactor;
     Rotation m_rotation;
@@ -260,7 +273,10 @@ private:
     QStandardItemModel* m_outlineModel;
     QStandardItemModel* m_propertiesModel;
 
+    bool checkDocument(const QString& filePath, Model::Document* document);
+
     void loadFallbackOutline();
+    void loadDocumentDefaults();
 
     void prepareDocument(Model::Document* document);
     void preparePages();
@@ -268,7 +284,7 @@ private:
     void prepareBackground();
 
     void prepareScene();
-    void prepareView(qreal changeLeft = 0.0, qreal changeTop = 0.0);
+    void prepareView(qreal changeLeft = 0.0, qreal changeTop = 0.0, int visiblePage = 0);
 
     void prepareThumbnailsScene();
 
@@ -287,4 +303,6 @@ private:
 
 };
 
+} // qpdfview
+
 #endif // DOCUMENTVIEW_H
