@@ -25,6 +25,9 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <QPair>
 
+namespace qpdfview
+{
+
 enum Rotation
 {
     RotateBy0 = 0,
@@ -39,7 +42,8 @@ enum RubberBandMode
     ModifiersMode = 0,
     CopyToClipboardMode = 1,
     AddAnnotationMode = 2,
-    NumberOfRubberBandModes = 3
+    ZoomToSelectionMode = 3,
+    NumberOfRubberBandModes = 4
 };
 
 enum LayoutMode
@@ -62,4 +66,6 @@ enum ScaleMode
 typedef QPair< int, QString > Jump;
 typedef QList< QPair< int, QString > > JumpList;
 
+} // qpdfview
+
 #endif // GLOBAL_H
