@@ -29,6 +29,9 @@ class QDialogButtonBox;
 class QTextBrowser;
 class QLineEdit;
 
+namespace qpdfview
+{
+
 class HelpDialog : public QDialog
 {
     Q_OBJECT
@@ -39,6 +42,7 @@ public:
 protected slots:
     void on_findPrevious_triggered();
     void on_findNext_triggered();
+    void on_search_textEdited();
 
 private:
     Q_DISABLE_COPY(HelpDialog)
@@ -53,4 +57,6 @@ private:
 
 };
 
+} // qpdfview
+
 #endif // HELPDIALOG_H
