@@ -1,6 +1,6 @@
 /*
 
-Copyright 2012-2013 Adam Reichold
+Copyright 2012-2014 Adam Reichold
 Copyright 2012 Michał Trybus
 Copyright 2012 Alexander Volkov
 
@@ -34,6 +34,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 class QCheckBox;
 class QGraphicsView;
+class QFileInfo;
 class QModelIndex;
 class QShortcut;
 class QTableView;
@@ -41,6 +42,9 @@ class QWidgetAction;
 
 #include "global.h"
 
+namespace qpdfview
+{
+
 class Settings;
 class DocumentView;
 class TabWidget;
@@ -49,6 +53,7 @@ class ComboBox;
 class SpinBox;
 class SearchLineEdit;
 class RecentlyUsedMenu;
+class RecentlyClosedMenu;
 class BookmarkMenu;
 class Database;
 
@@ -77,7 +82,6 @@ protected slots:
 
     void on_currentTab_documentChanged();
 
-    void on_currentTab_filePathChanged(const QString& filePath);
     void on_currentTab_numberOfPagesChaned(int numberOfPages);
     void on_currentTab_currentPageChanged(int currentPage);
 
@@ -85,10 +89,11 @@ protected slots:
 
     void on_currentTab_continuousModeChanged(bool continuousMode);
     void on_currentTab_layoutModeChanged(LayoutMode layoutMode);
+    void on_currentTab_rightToLeftModeChanged(bool rightToLeftMode);
     void on_currentTab_scaleModeChanged(ScaleMode scaleMode);
     void on_currentTab_scaleFactorChanged(qreal scaleFactor);
-    void on_currentTab_rotationChanged(Rotation rotation);
 
+    void on_currentTab_linkClicked(int page);
     void on_currentTab_linkClicked(const QString& filePath, int page);
 
     void on_currentTab_invertColorsChanged(bool invertColors);
@@ -142,9 +147,12 @@ protected slots:
     void on_twoPagesWithCoverPageMode_triggered(bool checked);
     void on_multiplePagesMode_triggered(bool checked);
 
+    void on_rightToLeftMode_triggered(bool checked);
+
     void on_zoomIn_triggered();
     void on_zoomOut_triggered();
     void on_originalSize_triggered();
+
     void on_fitToPageWidthMode_triggered(bool checked);
     void on_fitToPageSizeMode_triggered(bool checked);
 
@@ -164,6 +172,8 @@ protected slots:
     void on_closeAllTabs_triggered();
     void on_closeAllTabsButCurrentTab_triggered();
 
+    void on_recentlyClosed_tabActionTriggered(QAction* tabAction);
+
     void on_tabAction_triggered();
     void on_tabShortcut_activated();
 
@@ -173,9 +183,9 @@ protected slots:
     void on_removeBookmark_triggered();
     void on_removeAllBookmarks_triggered();
 
-    void on_bookmark_openTriggered(const QString& filePath);
-    void on_bookmark_openInNewTabTriggered(const QString& filePath);
-    void on_bookmark_jumpToPageTriggered(const QString& filePath, int page);
+    void on_bookmark_openTriggered(const QString& absoluteFilePath);
+    void on_bookmark_openInNewTabTriggered(const QString& absoluteFilePath);
+    void on_bookmark_jumpToPageTriggered(const QString& absoluteFilePath, int page);
 
     void on_contents_triggered();
     void on_about_triggered();
@@ -191,8 +201,10 @@ protected slots:
     void on_thumbnails_dockLocationChanged(Qt::DockWidgetArea area);
     void on_thumbnails_verticalScrollBar_valueChanged(int value);
 
-    void on_database_tabRestored(const QString& filePath, bool continousMode, LayoutMode layoutMode, ScaleMode scaleMode, qreal scaleFactor, Rotation rotation, int currentPage);
-    void on_database_bookmarkRestored(const QString& filePath, const JumpList& pages);
+    void on_database_tabRestored(const QString& absoluteFilePath, bool continuousMode, LayoutMode layoutMode, ScaleMode scaleMode, qreal scaleFactor, Rotation rotation, int currentPage);
+    void on_database_bookmarkRestored(const QString& absoluteFilePath, const JumpList& pages);
+
+    void on_saveDatabase_timeout();
 
 protected:
     void closeEvent(QCloseEvent* event);
@@ -214,14 +226,22 @@ private:
 
     bool senderIsCurrentTab() const;
 
-    void setWindowTitleForCurrentTab();
+    int addTab(DocumentView* tab);
+    void closeTab(DocumentView* tab);
 
+    QString tabTitle(const DocumentView* tab) const;
+
+    bool saveModifications(DocumentView* tab);
+
+    void setWindowTitleForCurrentTab();
     BookmarkMenu* bookmarkForCurrentTab() const;
 
+    void prepareDatabase();
+
     void saveTabs() const;
     void saveBookmarks() const;
 
-    bool saveModifications(DocumentView* tab);
+    QTimer* m_saveDatabaseTimer;
 
     SpinBox* m_currentPageSpinBox;
     QWidgetAction* m_currentPageAction;
@@ -269,6 +289,8 @@ private:
     QAction* m_twoPagesWithCoverPageModeAction;
     QAction* m_multiplePagesModeAction;
 
+    QAction* m_rightToLeftModeAction;
+
     QAction* m_zoomInAction;
     QAction* m_zoomOutAction;
     QAction* m_originalSizeAction;
@@ -342,6 +364,7 @@ private:
     QMenu* m_editMenu;
     QMenu* m_viewMenu;
     QMenu* m_tabsMenu;
+    RecentlyClosedMenu* m_recentlyClosedMenu;
     QMenu* m_bookmarksMenu;
     QMenu* m_helpMenu;
 
@@ -360,10 +383,10 @@ public:
     explicit MainWindowAdaptor(MainWindow* mainWindow);
 
 public slots:
-    bool open(const QString& filePath, int page = -1, const QRectF& highlight = QRectF(), bool quiet = false);
-    bool openInNewTab(const QString& filePath, int page = -1, const QRectF& highlight = QRectF(), bool quiet = false);
+    bool open(const QString& absoluteFilePath, int page = -1, const QRectF& highlight = QRectF(), bool quiet = false);
+    bool openInNewTab(const QString& absoluteFilePath, int page = -1, const QRectF& highlight = QRectF(), bool quiet = false);
 
-    bool jumpToPageOrOpenInNewTab(const QString& filePath, int page = -1, bool refreshBeforeJump = false, const QRectF& highlight = QRectF(), bool quiet = false);
+    bool jumpToPageOrOpenInNewTab(const QString& absoluteFilePath, int page = -1, bool refreshBeforeJump = false, const QRectF& highlight = QRectF(), bool quiet = false);
 
     Q_NOREPLY void startSearch(const QString& text);
 
@@ -376,4 +399,6 @@ private:
 
 #endif // WITH_DBUS
 
+} // qpdfview
+
 #endif // MAINWINDOW_H
