@@ -1,6 +1,7 @@
 /*
 
-Copyright 2012-2013 Adam Reichold
+Copyright 2012-2014 Adam Reichold
+Copyright 2014 Dorian Scholz
 
 This file is part of qpdfview.
 
@@ -21,10 +22,14 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "miscellaneous.h"
 
+#include <QApplication>
 #include <QMenu>
 #include <QMouseEvent>
 #include <QTimer>
 
+namespace qpdfview
+{
+
 GraphicsCompositionModeEffect::GraphicsCompositionModeEffect(QPainter::CompositionMode compositionMode, QObject* parent) : QGraphicsEffect(parent),
     m_compositionMode(compositionMode)
 {
@@ -46,6 +51,31 @@ TabBar::TabBar(QWidget* parent) : QTabBar(parent)
     setContextMenuPolicy(Qt::CustomContextMenu);
 }
 
+QSize TabBar::tabSizeHint(int index) const
+{
+    QSize size = QTabBar::tabSizeHint(index);
+
+    const TabWidget* tabWidget = qobject_cast< TabWidget* >(parentWidget());
+
+    if(tabWidget->spreadTabs())
+    {
+        switch(tabWidget->tabPosition())
+        {
+        default:
+        case QTabWidget::North:
+        case QTabWidget::South:
+            size.setWidth(qMax(width() / count(), size.width()));
+            break;
+        case QTabWidget::East:
+        case QTabWidget::West:
+            size.setHeight(qMax(height() / count(), size.height()));
+            break;
+        }
+    }
+
+    return size;
+}
+
 void TabBar::mousePressEvent(QMouseEvent* event)
 {
     QTabBar::mousePressEvent(event);
@@ -57,7 +87,8 @@ void TabBar::mousePressEvent(QMouseEvent* event)
 }
 
 TabWidget::TabWidget(QWidget* parent) : QTabWidget(parent),
-    m_tabBarPolicy(TabBarAsNeeded)
+    m_tabBarPolicy(TabBarAsNeeded),
+    m_spreadTabs(false)
 {
     setTabBar(new TabBar(this));
 
@@ -87,6 +118,19 @@ void TabWidget::setTabBarPolicy(TabWidget::TabBarPolicy tabBarPolicy)
     }
 }
 
+bool TabWidget::spreadTabs() const
+{
+    return m_spreadTabs;
+}
+
+void TabWidget::setSpreadTabs(bool spreadTabs)
+{
+    m_spreadTabs = spreadTabs;
+
+    QResizeEvent resizeEvent(tabBar()->size(), tabBar()->size());
+    QApplication::sendEvent(tabBar(), &resizeEvent);
+}
+
 void TabWidget::on_tabBar_customContextMenuRequested(const QPoint &pos)
 {
     emit tabContextMenuRequested(tabBar()->mapToGlobal(pos), tabBar()->tabAt(pos));
@@ -131,7 +175,10 @@ void TreeView::expandAll(const QModelIndex& index)
 {
     if(index.isValid())
     {
-        expand(index);
+        if(!isExpanded(index))
+        {
+            expand(index);
+        }
 
         for(int row = 0; row < model()->rowCount(index); ++row)
         {
@@ -148,7 +195,10 @@ void TreeView::collapseAll(const QModelIndex& index)
 {
     if(index.isValid())
     {
-        collapse(index);
+        if(isExpanded(index))
+        {
+            collapse(index);
+        }
 
         for(int row = 0; row < model()->rowCount(index); ++row)
         {
@@ -165,13 +215,11 @@ void TreeView::restoreExpansion(const QModelIndex& index)
 {
     if(index.isValid())
     {
-        if(model()->data(index, m_expansionRole).toBool())
+        const bool expanded = model()->data(index, m_expansionRole).toBool();
+
+        if(isExpanded(index) != expanded)
         {
-            expand(index);
-        }
-        else
-        {
-            collapse(index);
+            setExpanded(index, expanded);
         }
     }
 
@@ -321,3 +369,5 @@ void SearchLineEdit::on_returnPressed(const Qt::KeyboardModifiers& modifiers)
 
     emit searchInitiated(text(), modifiers == Qt::ShiftModifier);
 }
+
+} // qpdfview
