@@ -1,6 +1,7 @@
 /*
 
-Copyright 2012-2013 Adam Reichold
+Copyright 2012-2014 Adam Reichold
+Copyright 2014 Dorian Scholz
 
 This file is part of qpdfview.
 
@@ -29,6 +30,9 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <QSpinBox>
 #include <QTreeView>
 
+namespace qpdfview
+{
+
 // graphics composition mode effect
 
 class GraphicsCompositionModeEffect : public QGraphicsEffect
@@ -56,6 +60,8 @@ public:
     explicit TabBar(QWidget* parent = 0);
 
 protected:
+    QSize tabSizeHint(int index) const;
+
     void mousePressEvent(QMouseEvent* event);
 
 private:
@@ -82,6 +88,9 @@ public:
     TabBarPolicy tabBarPolicy() const;
     void setTabBarPolicy(TabBarPolicy tabBarPolicy);
 
+    bool spreadTabs() const;
+    void setSpreadTabs(bool spreadTabs);
+
 signals:
     void tabContextMenuRequested(const QPoint& globalPos, int index);
 
@@ -96,6 +105,7 @@ private:
     Q_DISABLE_COPY(TabWidget)
 
     TabBarPolicy m_tabBarPolicy;
+    bool m_spreadTabs;
 
 };
 
@@ -233,4 +243,6 @@ private:
 
 };
 
+} // qpdfview
+
 #endif // MISCELLANEOUS_H
