@@ -36,6 +36,9 @@ class QSizeF;
 
 #include "global.h"
 
+namespace qpdfview
+{
+
 namespace Model
 {
     struct Link
@@ -148,6 +151,12 @@ namespace Model
 
         virtual void loadFonts(QStandardItemModel* fontsModel) const { fontsModel->clear(); }
 
+        virtual bool wantsContinuousMode() const { return false; }
+        virtual bool wantsSinglePageMode() const { return false; }
+        virtual bool wantsTwoPagesMode() const { return false; }
+        virtual bool wantsTwoPagesWithCoverPageMode() const { return false; }
+        virtual bool wantsRightToLeftMode() const { return false; }
+
     };
 }
 
@@ -174,6 +183,8 @@ public:
 
 };
 
-Q_DECLARE_INTERFACE(Plugin, "local.qpdfview.Plugin")
+} // qpdfview
+
+Q_DECLARE_INTERFACE(qpdfview::Plugin, "local.qpdfview.Plugin")
 
 #endif // DOCUMENTMODEL_H
