@@ -1,6 +1,6 @@
 /*
 
-Copyright 2012-2013 Adam Reichold
+Copyright 2012-2014 Adam Reichold
 
 This file is part of qpdfview.
 
@@ -39,6 +39,9 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include "model.h"
 #include "rendertask.h"
 
+namespace qpdfview
+{
+
 Settings* PageItem::s_settings = 0;
 
 QCache< PageItem*, QPixmap > PageItem::s_cache;
@@ -125,21 +128,6 @@ void PageItem::paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget
     paintRubberBand(painter);
 }
 
-int PageItem::index() const
-{
-    return m_index;
-}
-
-const QSizeF& PageItem::size() const
-{
-    return m_size;
-}
-
-bool PageItem::invertColors()
-{
-    return m_invertColors;
-}
-
 void PageItem::setInvertColors(bool invertColors)
 {
     m_invertColors = invertColors;
@@ -147,11 +135,6 @@ void PageItem::setInvertColors(bool invertColors)
     refresh();
 }
 
-const QList< QRectF >& PageItem::highlights() const
-{
-    return m_highlights;
-}
-
 void PageItem::setHighlights(const QList< QRectF >& highlights)
 {
     m_highlights = highlights;
@@ -159,11 +142,6 @@ void PageItem::setHighlights(const QList< QRectF >& highlights)
     update();
 }
 
-RubberBandMode PageItem::rubberBandMode() const
-{
-    return m_rubberBandMode;
-}
-
 void PageItem::setRubberBandMode(RubberBandMode rubberBandMode)
 {
     if(m_rubberBandMode != rubberBandMode && rubberBandMode >= 0 && rubberBandMode < NumberOfRubberBandModes)
@@ -181,26 +159,6 @@ void PageItem::setRubberBandMode(RubberBandMode rubberBandMode)
     }
 }
 
-bool PageItem::showsAnnotationOverlay() const
-{
-    return !m_annotationOverlay.isEmpty();
-}
-
-bool PageItem::showsFormFieldOverlay() const
-{
-    return !m_formFieldOverlay.isEmpty();
-}
-
-int PageItem::resolutionX() const
-{
-    return m_resolutionX;
-}
-
-int PageItem::resolutionY() const
-{
-    return m_resolutionY;
-}
-
 void PageItem::setResolution(int resolutionX, int resolutionY)
 {
     if((m_resolutionX != resolutionX || m_resolutionY != resolutionY) && resolutionX > 0 && resolutionY > 0)
@@ -215,11 +173,6 @@ void PageItem::setResolution(int resolutionX, int resolutionY)
     }
 }
 
-qreal PageItem::devicePixelRatio() const
-{
-    return m_devicePixelRatio;
-}
-
 void PageItem::setDevicePixelRatio(qreal devicePixelRatio)
 {
     if(!qFuzzyCompare(m_devicePixelRatio, devicePixelRatio) && devicePixelRatio > 0.0)
@@ -233,11 +186,6 @@ void PageItem::setDevicePixelRatio(qreal devicePixelRatio)
     }
 }
 
-qreal PageItem::scaleFactor() const
-{
-    return m_scaleFactor;
-}
-
 void PageItem::setScaleFactor(qreal scaleFactor)
 {
     if(!qFuzzyCompare(m_scaleFactor, scaleFactor) && scaleFactor > 0.0)
@@ -251,11 +199,6 @@ void PageItem::setScaleFactor(qreal scaleFactor)
     }
 }
 
-Rotation PageItem::rotation() const
-{
-    return m_rotation;
-}
-
 void PageItem::setRotation(Rotation rotation)
 {
     if(m_rotation != rotation && rotation >= 0 && rotation < NumberOfRotations)
@@ -269,16 +212,6 @@ void PageItem::setRotation(Rotation rotation)
     }
 }
 
-const QTransform& PageItem::transform() const
-{
-    return m_transform;
-}
-
-const QTransform& PageItem::normalizedTransform() const
-{
-    return m_normalizedTransform;
-}
-
 void PageItem::refresh()
 {
     m_renderTask->cancel();
@@ -324,7 +257,10 @@ void PageItem::on_renderTask_finished()
 
 void PageItem::on_renderTask_imageReady(int resolutionX, int resolutionY, qreal devicePixelRatio, qreal scaleFactor, Rotation rotation, bool invertColors, bool prefetch, QImage image)
 {
-    if(m_resolutionX != resolutionX || m_resolutionY != resolutionY || !qFuzzyCompare(effectiveDevicePixelRatio(), devicePixelRatio) || !qFuzzyCompare(m_scaleFactor, scaleFactor) || m_rotation != rotation || m_invertColors != invertColors)
+    if(m_resolutionX != resolutionX || m_resolutionY != resolutionY
+            || !qFuzzyCompare(effectiveDevicePixelRatio(), devicePixelRatio)
+            || !qFuzzyCompare(m_scaleFactor, scaleFactor) || m_rotation != rotation
+            || m_invertColors != invertColors)
     {
         return;
     }
@@ -498,19 +434,25 @@ void PageItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
     // rubber band
 
     if(m_rubberBandMode == ModifiersMode && !m_presentationMode
-            && (event->modifiers() == s_settings->pageItem().copyToClipboardModifiers() || event->modifiers() == s_settings->pageItem().addAnnotationModifiers())
-            && event->button() == Qt::LeftButton)
+            && (event->modifiers() == Qt::NoModifier
+                || event->modifiers() == s_settings->pageItem().copyToClipboardModifiers()
+                || event->modifiers() == s_settings->pageItem().addAnnotationModifiers())
+            && (event->button() == Qt::LeftButton || event->button() == Qt::MidButton))
     {
         setCursor(Qt::CrossCursor);
 
-        if(event->modifiers() == s_settings->pageItem().copyToClipboardModifiers())
+        if(event->modifiers() == s_settings->pageItem().copyToClipboardModifiers() && event->button() == Qt::LeftButton)
         {
             m_rubberBandMode = CopyToClipboardMode;
         }
-        else if(event->modifiers() == s_settings->pageItem().addAnnotationModifiers())
+        else if(event->modifiers() == s_settings->pageItem().addAnnotationModifiers() && event->button() == Qt::LeftButton)
         {
             m_rubberBandMode = AddAnnotationMode;
         }
+        else if(event->modifiers() == Qt::NoModifier && event->button() == Qt::MidButton)
+        {
+            m_rubberBandMode = ZoomToSelectionMode;
+        }
     }
 
     if(m_rubberBandMode != ModifiersMode)
@@ -525,7 +467,8 @@ void PageItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
         return;
     }
 
-    if(event->modifiers() == Qt::NoModifier && event->button() == Qt::LeftButton)
+    if(event->modifiers() == Qt::NoModifier
+            && (event->button() == Qt::LeftButton || event->button() == Qt::MidButton))
     {
         // links
 
@@ -539,7 +482,7 @@ void PageItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
                 {
                     if(link->urlOrFileName.isNull())
                     {
-                        emit linkClicked(link->page, link->left, link->top);
+                        emit linkClicked(event->button() == Qt::MidButton, link->page, link->left, link->top);
                     }
                     else
                     {
@@ -559,7 +502,7 @@ void PageItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
             }
         }
 
-        if(m_presentationMode)
+        if(event->button() == Qt::MidButton || m_presentationMode)
         {
             event->ignore();
             return;
@@ -605,7 +548,9 @@ void PageItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
 
 void PageItem::mouseDoubleClickEvent(QGraphicsSceneMouseEvent* event)
 {
-    emit sourceRequested(m_index + 1, m_transform.inverted().map(event->pos()));
+    emit editSourceRequested(m_index + 1, m_transform.inverted().map(event->pos()));
+
+    event->accept();
 }
 
 void PageItem::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
@@ -642,6 +587,10 @@ void PageItem::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
         {
             addAnnotation(event->screenPos());
         }
+        else if(m_rubberBandMode == ZoomToSelectionMode)
+        {
+            emit zoomToSelectionRequested(m_index + 1, m_normalizedTransform.inverted().mapRect(m_rubberBand));
+        }
 
         m_rubberBandMode = ModifiersMode;
         m_rubberBand = QRectF();
@@ -710,18 +659,28 @@ void PageItem::copyToClipboard(const QPoint& screenPos)
     QMenu menu;
 
     const QAction* copyTextAction = menu.addAction(tr("Copy &text"));
+    QAction* useTextAsSelectionAction = menu.addAction(tr("Use text as &selection"));
     const QAction* copyImageAction = menu.addAction(tr("Copy &image"));
     const QAction* saveImageToFileAction = menu.addAction(tr("Save image to &file..."));
 
+    useTextAsSelectionAction->setVisible(QApplication::clipboard()->supportsSelection());
+
     const QAction* action = menu.exec(screenPos);
 
-    if(action == copyTextAction)
+    if(action == copyTextAction || action == useTextAsSelectionAction)
     {
         const QString text = m_page->text(m_transform.inverted().mapRect(m_rubberBand));
 
         if(!text.isEmpty())
         {
-            QApplication::clipboard()->setText(text);
+            if(action == copyTextAction)
+            {
+                QApplication::clipboard()->setText(text);
+            }
+            else
+            {
+                QApplication::clipboard()->setText(text, QClipboard::Selection);
+            }
         }
     }
     else if(action == copyImageAction || action == saveImageToFileAction)
@@ -952,7 +911,7 @@ void PageItem::setProxyGeometry(Model::FormField* formField, QGraphicsProxyWidge
 
 qreal PageItem::effectiveDevicePixelRatio()
 {
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+#if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
 
     return s_settings->pageItem().useDevicePixelRatio() ? m_devicePixelRatio : 1.0;
 
@@ -1216,11 +1175,6 @@ void ThumbnailItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* opt
     }
 }
 
-bool ThumbnailItem::isCurrent() const
-{
-    return m_current;
-}
-
 void ThumbnailItem::setCurrent(bool current)
 {
     if(m_current != current)
@@ -1231,9 +1185,18 @@ void ThumbnailItem::setCurrent(bool current)
     }
 }
 
-void ThumbnailItem::mousePressEvent(QGraphicsSceneMouseEvent*)
+void ThumbnailItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
 {
-    emit linkClicked(index() + 1);
+    if(event->modifiers() == Qt::NoModifier
+            && (event->button() == Qt::LeftButton || event->button() == Qt::MidButton))
+    {
+        emit linkClicked(event->button() == Qt::MidButton, index() + 1);
+
+        event->accept();
+        return;
+    }
+
+    event->ignore();
 }
 
 void ThumbnailItem::mouseDoubleClickEvent(QGraphicsSceneMouseEvent*)
@@ -1297,3 +1260,5 @@ void ThumbnailItem::loadInteractiveElements()
 
     setToolTip(QString("%1 mm x %2 mm%3").arg(width, 0, 'f', 1).arg(height, 0, 'f', 1).arg(paperSize));
 }
+
+} // qpdfview
