@@ -40,641 +40,10 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include "annotationwidgets.h"
 #include "formfieldwidgets.h"
 
-Model::PdfAnnotation::PdfAnnotation(QMutex* mutex, Poppler::Annotation* annotation) : Annotation(),
-    m_mutex(mutex),
-    m_annotation(annotation)
+namespace
 {
-}
 
-Model::PdfAnnotation::~PdfAnnotation()
-{
-    delete m_annotation;
-}
-
-QRectF Model::PdfAnnotation::boundary() const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    return m_annotation->boundary().normalized();
-}
-
-QString Model::PdfAnnotation::contents() const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    return m_annotation->contents();
-}
-
-QWidget* Model::PdfAnnotation::createWidget()
-{
-    QWidget* widget = 0;
-
-    if(m_annotation->subType() == Poppler::Annotation::AText || m_annotation->subType() == Poppler::Annotation::AHighlight)
-    {
-        widget = new AnnotationWidget(m_mutex, m_annotation);
-
-        connect(widget, SIGNAL(wasModified()), SIGNAL(wasModified()));
-    }
-    else if(m_annotation->subType() == Poppler::Annotation::AFileAttachment)
-    {
-        widget = new FileAttachmentAnnotationWidget(m_mutex, static_cast< Poppler::FileAttachmentAnnotation* >(m_annotation));
-    }
-
-    connect(this, SIGNAL(destroyed()), widget, SLOT(deleteLater()));
-
-    return widget;
-}
-
-Model::PdfFormField::PdfFormField(QMutex* mutex, Poppler::FormField* formField) : FormField(),
-    m_mutex(mutex),
-    m_formField(formField)
-{
-}
-
-Model::PdfFormField::~PdfFormField()
-{
-    delete m_formField;
-}
-
-QRectF Model::PdfFormField::boundary() const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    return m_formField->rect().normalized();
-}
-
-QString Model::PdfFormField::name() const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    return m_formField->name();
-}
-
-QWidget* Model::PdfFormField::createWidget()
-{
-    QWidget* widget = 0;
-
-    if(m_formField->type() == Poppler::FormField::FormText)
-    {
-        Poppler::FormFieldText* formFieldText = static_cast< Poppler::FormFieldText* >(m_formField);
-
-        if(formFieldText->textType() == Poppler::FormFieldText::Normal)
-        {
-            widget = new NormalTextFieldWidget(m_mutex, formFieldText);
-        }
-        else if(formFieldText->textType() == Poppler::FormFieldText::Multiline)
-        {
-            widget = new MultilineTextFieldWidget(m_mutex, formFieldText);
-        }
-    }
-    else if(m_formField->type() == Poppler::FormField::FormChoice)
-    {
-        Poppler::FormFieldChoice* formFieldChoice = static_cast< Poppler::FormFieldChoice* >(m_formField);
-
-        if(formFieldChoice->choiceType() == Poppler::FormFieldChoice::ComboBox)
-        {
-            widget = new ComboBoxChoiceFieldWidget(m_mutex, formFieldChoice);
-        }
-        else if(formFieldChoice->choiceType() == Poppler::FormFieldChoice::ListBox)
-        {
-            widget = new ListBoxChoiceFieldWidget(m_mutex, formFieldChoice);
-        }
-    }
-    else if(m_formField->type() == Poppler::FormField::FormButton)
-    {
-        Poppler::FormFieldButton* formFieldButton = static_cast< Poppler::FormFieldButton* >(m_formField);
-
-        if(formFieldButton->buttonType() == Poppler::FormFieldButton::CheckBox)
-        {
-            widget = new CheckBoxChoiceFieldWidget(m_mutex, formFieldButton);
-        }
-        else if(formFieldButton->buttonType() == Poppler::FormFieldButton::Radio)
-        {
-            widget = new RadioChoiceFieldWidget(m_mutex, formFieldButton);
-        }
-    }
-
-    connect(widget, SIGNAL(wasModified()), SIGNAL(wasModified()));
-
-    return widget;
-}
-
-Model::PdfPage::PdfPage(QMutex* mutex, Poppler::Page* page) :
-    m_mutex(mutex),
-    m_page(page)
-{
-}
-
-Model::PdfPage::~PdfPage()
-{
-    delete m_page;
-}
-
-QSizeF Model::PdfPage::size() const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    return m_page->pageSizeF();
-}
-
-QImage Model::PdfPage::render(qreal horizontalResolution, qreal verticalResolution, Rotation rotation, const QRect& boundingRect) const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    double xres;
-    double yres;
-
-    switch(rotation)
-    {
-    default:
-    case RotateBy0:
-    case RotateBy180:
-        xres = horizontalResolution;
-        yres = verticalResolution;
-        break;
-    case RotateBy90:
-    case RotateBy270:
-        xres = verticalResolution;
-        yres = horizontalResolution;
-        break;
-    }
-
-    Poppler::Page::Rotation rotate;
-
-    switch(rotation)
-    {
-    default:
-    case RotateBy0:
-        rotate = Poppler::Page::Rotate0;
-        break;
-    case RotateBy90:
-        rotate = Poppler::Page::Rotate90;
-        break;
-    case RotateBy180:
-        rotate = Poppler::Page::Rotate180;
-        break;
-    case RotateBy270:
-        rotate = Poppler::Page::Rotate270;
-        break;
-    }
-
-    int x = -1;
-    int y = -1;
-    int w = -1;
-    int h = -1;
-
-    if(!boundingRect.isNull())
-    {
-        x = boundingRect.x();
-        y = boundingRect.y();
-        w = boundingRect.width();
-        h = boundingRect.height();
-    }
-
-    return m_page->renderToImage(xres, yres, x, y, w, h, rotate);
-}
-
-QList< Model::Link* > Model::PdfPage::links() const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    QList< Link* > links;
-
-    foreach(const Poppler::Link* link, m_page->links())
-    {
-        if(link->linkType() == Poppler::Link::Goto)
-        {
-            const Poppler::LinkGoto* linkGoto = static_cast< const Poppler::LinkGoto* >(link);
-
-            const QRectF boundary = linkGoto->linkArea().normalized();
-            const int page = linkGoto->destination().pageNumber();
-
-            qreal left = linkGoto->destination().isChangeLeft() ? linkGoto->destination().left() : 0.0;
-            qreal top = linkGoto->destination().isChangeTop() ? linkGoto->destination().top() : 0.0;
-
-            left = left >= 0.0 ? left : 0.0;
-            left = left <= 1.0 ? left : 1.0;
-
-            top = top >= 0.0 ? top : 0.0;
-            top = top <= 1.0 ? top : 1.0;
-
-            if(linkGoto->isExternal())
-            {
-                links.append(new Link(boundary, linkGoto->fileName(), page));
-            }
-            else
-            {
-                links.append(new Link(boundary, page, left, top));
-            }
-        }
-        else if(link->linkType() == Poppler::Link::Browse)
-        {
-            const Poppler::LinkBrowse* linkBrowse = static_cast< const Poppler::LinkBrowse* >(link);
-
-            const QRectF boundary = linkBrowse->linkArea().normalized();
-            const QString url = linkBrowse->url();
-
-            links.append(new Link(boundary, url));
-        }
-        else if(link->linkType() == Poppler::Link::Execute)
-        {
-            const Poppler::LinkExecute* linkExecute = static_cast< const Poppler::LinkExecute* >(link);
-
-            const QRectF boundary = linkExecute->linkArea().normalized();
-            const QString url = linkExecute->fileName();
-
-            links.append(new Link(boundary, url));
-        }
-
-        delete link;
-    }
-
-    return links;
-}
-
-QString Model::PdfPage::text(const QRectF& rect) const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    return m_page->text(rect);
-}
-
-QList< QRectF > Model::PdfPage::search(const QString& text, bool matchCase) const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    QList< QRectF > results;
-
-#if defined(HAS_POPPLER_22)
-
-    results = m_page->search(text, matchCase ? Poppler::Page::CaseSensitive : Poppler::Page::CaseInsensitive);
-
-#elif defined(HAS_POPPLER_14)
-
-    double left = 0.0, top = 0.0, right = 0.0, bottom = 0.0;
-
-    while(m_page->search(text, left, top, right, bottom, Poppler::Page::NextResult, matchCase ? Poppler::Page::CaseSensitive : Poppler::Page::CaseInsensitive))
-    {
-        QRectF rect;
-        rect.setLeft(left);
-        rect.setTop(top);
-        rect.setRight(right);
-        rect.setBottom(bottom);
-
-        results.append(rect);
-    }
-
-#else
-
-    QRectF rect;
-
-    while(m_page->search(text, rect, Poppler::Page::NextResult, matchCase ? Poppler::Page::CaseSensitive : Poppler::Page::CaseInsensitive))
-    {
-        results.append(rect);
-    }
-
-#endif
-
-    return results;
-}
-
-QList< Model::Annotation* > Model::PdfPage::annotations() const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    QList< Annotation* > annotations;
-
-    foreach(Poppler::Annotation* annotation, m_page->annotations())
-    {
-        if(annotation->subType() == Poppler::Annotation::AText || annotation->subType() == Poppler::Annotation::AHighlight || annotation->subType() == Poppler::Annotation::AFileAttachment)
-        {
-            annotations.append(new PdfAnnotation(m_mutex, annotation));
-            continue;
-        }
-
-        delete annotation;
-    }
-
-    return annotations;
-}
-
-bool Model::PdfPage::canAddAndRemoveAnnotations() const
-{
-#ifdef HAS_POPPLER_20
-
-    return true;
-
-#else
-
-    QMessageBox::information(0, tr("Information"), tr("Version 0.20.1 or higher of the Poppler library is required to add or remove annotations."));
-
-    return false;
-
-#endif // HAS_POPPLER_20
-}
-
-Model::Annotation* Model::PdfPage::addTextAnnotation(const QRectF& boundary, const QColor& color)
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-#ifdef HAS_POPPLER_20
-
-    Poppler::Annotation::Style style;
-    style.setColor(color);
-
-    Poppler::Annotation::Popup popup;
-    popup.setFlags(Poppler::Annotation::Hidden | Poppler::Annotation::ToggleHidingOnMouse);
-
-    Poppler::Annotation* annotation = new Poppler::TextAnnotation(Poppler::TextAnnotation::Linked);
-
-    annotation->setBoundary(boundary);
-    annotation->setStyle(style);
-    annotation->setPopup(popup);
-
-    m_page->addAnnotation(annotation);
-
-    return new PdfAnnotation(m_mutex, annotation);
-
-#else
-
-    Q_UNUSED(boundary);
-    Q_UNUSED(color);
-
-    return 0;
-
-#endif // HAS_POPPLER_20
-}
-
-Model::Annotation* Model::PdfPage::addHighlightAnnotation(const QRectF& boundary, const QColor& color)
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-#ifdef HAS_POPPLER_20
-
-    Poppler::Annotation::Style style;
-    style.setColor(color);
-
-    Poppler::Annotation::Popup popup;
-    popup.setFlags(Poppler::Annotation::Hidden | Poppler::Annotation::ToggleHidingOnMouse);
-
-    Poppler::HighlightAnnotation* annotation = new Poppler::HighlightAnnotation();
-
-    Poppler::HighlightAnnotation::Quad quad;
-    quad.points[0] = boundary.topLeft();
-    quad.points[1] = boundary.topRight();
-    quad.points[2] = boundary.bottomRight();
-    quad.points[3] = boundary.bottomLeft();
-
-    annotation->setHighlightQuads(QList< Poppler::HighlightAnnotation::Quad >() << quad);
-
-    annotation->setBoundary(boundary);
-    annotation->setStyle(style);
-    annotation->setPopup(popup);
-
-    m_page->addAnnotation(annotation);
-
-    return new PdfAnnotation(m_mutex, annotation);
-
-#else
-
-    Q_UNUSED(boundary);
-    Q_UNUSED(color);
-
-    return 0;
-
-#endif // HAS_POPPLER_20
-
-}
-
-void Model::PdfPage::removeAnnotation(Annotation* annotation)
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-#ifdef HAS_POPPLER_20
-
-    PdfAnnotation* pdfAnnotation = static_cast< PdfAnnotation* >(annotation);
-
-    m_page->removeAnnotation(pdfAnnotation->m_annotation);
-    pdfAnnotation->m_annotation = 0;
-
-#else
-
-    Q_UNUSED(annotation);
-
-#endif // HAS_POPPLER_20
-}
-
-QList< Model::FormField* > Model::PdfPage::formFields() const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    QList< FormField* > formFields;
-
-    foreach(Poppler::FormField* formField, m_page->formFields())
-    {
-        if(!formField->isVisible() || formField->isReadOnly())
-        {
-            delete formField;
-            continue;
-        }
-
-        if(formField->type() == Poppler::FormField::FormText)
-        {
-            Poppler::FormFieldText* formFieldText = static_cast< Poppler::FormFieldText* >(formField);
-
-            if(formFieldText->textType() == Poppler::FormFieldText::Normal || formFieldText->textType() == Poppler::FormFieldText::Multiline)
-            {
-                formFields.append(new PdfFormField(m_mutex, formField));
-                continue;
-            }
-        }
-        else if(formField->type() == Poppler::FormField::FormChoice)
-        {
-            Poppler::FormFieldChoice* formFieldChoice = static_cast< Poppler::FormFieldChoice* >(formField);
-
-            if(formFieldChoice->choiceType() == Poppler::FormFieldChoice::ListBox || formFieldChoice->choiceType() == Poppler::FormFieldChoice::ComboBox)
-            {
-                formFields.append(new PdfFormField(m_mutex, formField));
-                continue;
-            }
-        }
-        else if(formField->type() == Poppler::FormField::FormButton)
-        {
-            Poppler::FormFieldButton* formFieldButton = static_cast< Poppler::FormFieldButton* >(formField);
-
-            if(formFieldButton->buttonType() == Poppler::FormFieldButton::CheckBox || formFieldButton->buttonType() == Poppler::FormFieldButton::Radio)
-            {
-                formFields.append(new PdfFormField(m_mutex, formField));
-                continue;
-            }
-        }
-
-        delete formField;
-    }
-
-    return formFields;
-}
-
-Model::PdfDocument::PdfDocument(Poppler::Document* document) :
-    m_mutex(),
-    m_document(document)
-{
-}
-
-Model::PdfDocument::~PdfDocument()
-{
-    delete m_document;
-}
-
-int Model::PdfDocument::numberOfPages() const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    return m_document->numPages();
-}
-
-Model::Page* Model::PdfDocument::page(int index) const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    Poppler::Page* page = m_document->page(index);
-
-    return page != 0 ? new PdfPage(&m_mutex, page) : 0;
-}
-
-bool Model::PdfDocument::isLocked() const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    return m_document->isLocked();
-}
-
-bool Model::PdfDocument::unlock(const QString& password)
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    return m_document->unlock(password.toLatin1(), password.toLatin1());
-}
-
-QStringList Model::PdfDocument::saveFilter() const
-{
-    return QStringList() << "Portable document format (*.pdf)";
-}
-
-bool Model::PdfDocument::canSave() const
-{
-    return true;
-}
-
-bool Model::PdfDocument::save(const QString& filePath, bool withChanges) const
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    QScopedPointer< Poppler::PDFConverter > pdfConverter(m_document->pdfConverter());
-
-    pdfConverter->setOutputFileName(filePath);
-
-    if(withChanges)
-    {
-        pdfConverter->setPDFOptions(pdfConverter->pdfOptions() | Poppler::PDFConverter::WithChanges);
-    }
-
-    return pdfConverter->convert();
-}
-
-bool Model::PdfDocument::canBePrintedUsingCUPS() const
-{
-    return true;
-}
-
-void Model::PdfDocument::setPaperColor(const QColor& paperColor)
-{
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    m_document->setPaperColor(paperColor);
-}
-
-static void loadOutline(Poppler::Document* document, const QDomNode& node, QStandardItem* parent)
+void loadOutline(Poppler::Document* document, const QDomNode& node, QStandardItem* parent)
 {
     const QDomElement element = node.toElement();
 
@@ -747,6 +116,645 @@ static void loadOutline(Poppler::Document* document, const QDomNode& node, QStan
     }
 }
 
+} // anonymous
+
+namespace qpdfview
+{
+
+namespace Model
+{
+
+PdfAnnotation::PdfAnnotation(QMutex* mutex, Poppler::Annotation* annotation) : Annotation(),
+    m_mutex(mutex),
+    m_annotation(annotation)
+{
+}
+
+PdfAnnotation::~PdfAnnotation()
+{
+    delete m_annotation;
+}
+
+QRectF PdfAnnotation::boundary() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    return m_annotation->boundary().normalized();
+}
+
+QString PdfAnnotation::contents() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    return m_annotation->contents();
+}
+
+QWidget* PdfAnnotation::createWidget()
+{
+    QWidget* widget = 0;
+
+    if(m_annotation->subType() == Poppler::Annotation::AText || m_annotation->subType() == Poppler::Annotation::AHighlight)
+    {
+        widget = new AnnotationWidget(m_mutex, m_annotation);
+
+        connect(widget, SIGNAL(wasModified()), SIGNAL(wasModified()));
+    }
+    else if(m_annotation->subType() == Poppler::Annotation::AFileAttachment)
+    {
+        widget = new FileAttachmentAnnotationWidget(m_mutex, static_cast< Poppler::FileAttachmentAnnotation* >(m_annotation));
+    }
+
+    connect(this, SIGNAL(destroyed()), widget, SLOT(deleteLater()));
+
+    return widget;
+}
+
+PdfFormField::PdfFormField(QMutex* mutex, Poppler::FormField* formField) : FormField(),
+    m_mutex(mutex),
+    m_formField(formField)
+{
+}
+
+PdfFormField::~PdfFormField()
+{
+    delete m_formField;
+}
+
+QRectF PdfFormField::boundary() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    return m_formField->rect().normalized();
+}
+
+QString PdfFormField::name() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    return m_formField->name();
+}
+
+QWidget* PdfFormField::createWidget()
+{
+    QWidget* widget = 0;
+
+    if(m_formField->type() == Poppler::FormField::FormText)
+    {
+        Poppler::FormFieldText* formFieldText = static_cast< Poppler::FormFieldText* >(m_formField);
+
+        if(formFieldText->textType() == Poppler::FormFieldText::Normal)
+        {
+            widget = new NormalTextFieldWidget(m_mutex, formFieldText);
+        }
+        else if(formFieldText->textType() == Poppler::FormFieldText::Multiline)
+        {
+            widget = new MultilineTextFieldWidget(m_mutex, formFieldText);
+        }
+    }
+    else if(m_formField->type() == Poppler::FormField::FormChoice)
+    {
+        Poppler::FormFieldChoice* formFieldChoice = static_cast< Poppler::FormFieldChoice* >(m_formField);
+
+        if(formFieldChoice->choiceType() == Poppler::FormFieldChoice::ComboBox)
+        {
+            widget = new ComboBoxChoiceFieldWidget(m_mutex, formFieldChoice);
+        }
+        else if(formFieldChoice->choiceType() == Poppler::FormFieldChoice::ListBox)
+        {
+            widget = new ListBoxChoiceFieldWidget(m_mutex, formFieldChoice);
+        }
+    }
+    else if(m_formField->type() == Poppler::FormField::FormButton)
+    {
+        Poppler::FormFieldButton* formFieldButton = static_cast< Poppler::FormFieldButton* >(m_formField);
+
+        if(formFieldButton->buttonType() == Poppler::FormFieldButton::CheckBox)
+        {
+            widget = new CheckBoxChoiceFieldWidget(m_mutex, formFieldButton);
+        }
+        else if(formFieldButton->buttonType() == Poppler::FormFieldButton::Radio)
+        {
+            widget = new RadioChoiceFieldWidget(m_mutex, formFieldButton);
+        }
+    }
+
+    connect(widget, SIGNAL(wasModified()), SIGNAL(wasModified()));
+
+    return widget;
+}
+
+PdfPage::PdfPage(QMutex* mutex, Poppler::Page* page) :
+    m_mutex(mutex),
+    m_page(page)
+{
+}
+
+PdfPage::~PdfPage()
+{
+    delete m_page;
+}
+
+QSizeF PdfPage::size() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    return m_page->pageSizeF();
+}
+
+QImage PdfPage::render(qreal horizontalResolution, qreal verticalResolution, Rotation rotation, const QRect& boundingRect) const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    double xres;
+    double yres;
+
+    switch(rotation)
+    {
+    default:
+    case RotateBy0:
+    case RotateBy180:
+        xres = horizontalResolution;
+        yres = verticalResolution;
+        break;
+    case RotateBy90:
+    case RotateBy270:
+        xres = verticalResolution;
+        yres = horizontalResolution;
+        break;
+    }
+
+    Poppler::Page::Rotation rotate;
+
+    switch(rotation)
+    {
+    default:
+    case RotateBy0:
+        rotate = Poppler::Page::Rotate0;
+        break;
+    case RotateBy90:
+        rotate = Poppler::Page::Rotate90;
+        break;
+    case RotateBy180:
+        rotate = Poppler::Page::Rotate180;
+        break;
+    case RotateBy270:
+        rotate = Poppler::Page::Rotate270;
+        break;
+    }
+
+    int x = -1;
+    int y = -1;
+    int w = -1;
+    int h = -1;
+
+    if(!boundingRect.isNull())
+    {
+        x = boundingRect.x();
+        y = boundingRect.y();
+        w = boundingRect.width();
+        h = boundingRect.height();
+    }
+
+    return m_page->renderToImage(xres, yres, x, y, w, h, rotate);
+}
+
+QList< Link* > PdfPage::links() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    QList< Link* > links;
+
+    foreach(const Poppler::Link* link, m_page->links())
+    {
+        const QRectF boundary = link->linkArea().normalized();
+
+        if(link->linkType() == Poppler::Link::Goto)
+        {
+            const Poppler::LinkGoto* linkGoto = static_cast< const Poppler::LinkGoto* >(link);
+
+            const int page = linkGoto->destination().pageNumber();
+
+            qreal left = linkGoto->destination().isChangeLeft() ? linkGoto->destination().left() : 0.0;
+            qreal top = linkGoto->destination().isChangeTop() ? linkGoto->destination().top() : 0.0;
+
+            left = left >= 0.0 ? left : 0.0;
+            left = left <= 1.0 ? left : 1.0;
+
+            top = top >= 0.0 ? top : 0.0;
+            top = top <= 1.0 ? top : 1.0;
+
+            if(linkGoto->isExternal())
+            {
+                links.append(new Link(boundary, linkGoto->fileName(), page));
+            }
+            else
+            {
+                links.append(new Link(boundary, page, left, top));
+            }
+        }
+        else if(link->linkType() == Poppler::Link::Browse)
+        {
+            const Poppler::LinkBrowse* linkBrowse = static_cast< const Poppler::LinkBrowse* >(link);
+            const QString url = linkBrowse->url();
+
+            links.append(new Link(boundary, url));
+        }
+        else if(link->linkType() == Poppler::Link::Execute)
+        {
+            const Poppler::LinkExecute* linkExecute = static_cast< const Poppler::LinkExecute* >(link);
+            const QString url = linkExecute->fileName();
+
+            links.append(new Link(boundary, url));
+        }
+
+        delete link;
+    }
+
+    return links;
+}
+
+QString PdfPage::text(const QRectF& rect) const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    return m_page->text(rect);
+}
+
+QList< QRectF > PdfPage::search(const QString& text, bool matchCase) const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    QList< QRectF > results;
+
+#if defined(HAS_POPPLER_22)
+
+    results = m_page->search(text, matchCase ? Poppler::Page::CaseSensitive : Poppler::Page::CaseInsensitive);
+
+#elif defined(HAS_POPPLER_14)
+
+    double left = 0.0, top = 0.0, right = 0.0, bottom = 0.0;
+
+    while(m_page->search(text, left, top, right, bottom, Poppler::Page::NextResult, matchCase ? Poppler::Page::CaseSensitive : Poppler::Page::CaseInsensitive))
+    {
+        QRectF rect;
+        rect.setLeft(left);
+        rect.setTop(top);
+        rect.setRight(right);
+        rect.setBottom(bottom);
+
+        results.append(rect);
+    }
+
+#else
+
+    QRectF rect;
+
+    while(m_page->search(text, rect, Poppler::Page::NextResult, matchCase ? Poppler::Page::CaseSensitive : Poppler::Page::CaseInsensitive))
+    {
+        results.append(rect);
+    }
+
+#endif
+
+    return results;
+}
+
+QList< Annotation* > PdfPage::annotations() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    QList< Annotation* > annotations;
+
+    foreach(Poppler::Annotation* annotation, m_page->annotations())
+    {
+        if(annotation->subType() == Poppler::Annotation::AText || annotation->subType() == Poppler::Annotation::AHighlight || annotation->subType() == Poppler::Annotation::AFileAttachment)
+        {
+            annotations.append(new PdfAnnotation(m_mutex, annotation));
+            continue;
+        }
+
+        delete annotation;
+    }
+
+    return annotations;
+}
+
+bool Model::PdfPage::canAddAndRemoveAnnotations() const
+{
+#ifdef HAS_POPPLER_20
+
+    return true;
+
+#else
+
+    QMessageBox::information(0, tr("Information"), tr("Version 0.20.1 or higher of the Poppler library is required to add or remove annotations."));
+
+    return false;
+
+#endif // HAS_POPPLER_20
+}
+
+Annotation* PdfPage::addTextAnnotation(const QRectF& boundary, const QColor& color)
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+#ifdef HAS_POPPLER_20
+
+    Poppler::Annotation::Style style;
+    style.setColor(color);
+
+    Poppler::Annotation::Popup popup;
+    popup.setFlags(Poppler::Annotation::Hidden | Poppler::Annotation::ToggleHidingOnMouse);
+
+    Poppler::Annotation* annotation = new Poppler::TextAnnotation(Poppler::TextAnnotation::Linked);
+
+    annotation->setBoundary(boundary);
+    annotation->setStyle(style);
+    annotation->setPopup(popup);
+
+    m_page->addAnnotation(annotation);
+
+    return new PdfAnnotation(m_mutex, annotation);
+
+#else
+
+    Q_UNUSED(boundary);
+    Q_UNUSED(color);
+
+    return 0;
+
+#endif // HAS_POPPLER_20
+}
+
+Annotation* PdfPage::addHighlightAnnotation(const QRectF& boundary, const QColor& color)
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+#ifdef HAS_POPPLER_20
+
+    Poppler::Annotation::Style style;
+    style.setColor(color);
+
+    Poppler::Annotation::Popup popup;
+    popup.setFlags(Poppler::Annotation::Hidden | Poppler::Annotation::ToggleHidingOnMouse);
+
+    Poppler::HighlightAnnotation* annotation = new Poppler::HighlightAnnotation();
+
+    Poppler::HighlightAnnotation::Quad quad;
+    quad.points[0] = boundary.topLeft();
+    quad.points[1] = boundary.topRight();
+    quad.points[2] = boundary.bottomRight();
+    quad.points[3] = boundary.bottomLeft();
+
+    annotation->setHighlightQuads(QList< Poppler::HighlightAnnotation::Quad >() << quad);
+
+    annotation->setBoundary(boundary);
+    annotation->setStyle(style);
+    annotation->setPopup(popup);
+
+    m_page->addAnnotation(annotation);
+
+    return new PdfAnnotation(m_mutex, annotation);
+
+#else
+
+    Q_UNUSED(boundary);
+    Q_UNUSED(color);
+
+    return 0;
+
+#endif // HAS_POPPLER_20
+
+}
+
+void PdfPage::removeAnnotation(Annotation* annotation)
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+#ifdef HAS_POPPLER_20
+
+    PdfAnnotation* pdfAnnotation = static_cast< PdfAnnotation* >(annotation);
+
+    m_page->removeAnnotation(pdfAnnotation->m_annotation);
+    pdfAnnotation->m_annotation = 0;
+
+#else
+
+    Q_UNUSED(annotation);
+
+#endif // HAS_POPPLER_20
+}
+
+QList< FormField* > PdfPage::formFields() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    QList< FormField* > formFields;
+
+    foreach(Poppler::FormField* formField, m_page->formFields())
+    {
+        if(!formField->isVisible() || formField->isReadOnly())
+        {
+            delete formField;
+            continue;
+        }
+
+        if(formField->type() == Poppler::FormField::FormText)
+        {
+            Poppler::FormFieldText* formFieldText = static_cast< Poppler::FormFieldText* >(formField);
+
+            if(formFieldText->textType() == Poppler::FormFieldText::Normal || formFieldText->textType() == Poppler::FormFieldText::Multiline)
+            {
+                formFields.append(new PdfFormField(m_mutex, formField));
+                continue;
+            }
+        }
+        else if(formField->type() == Poppler::FormField::FormChoice)
+        {
+            Poppler::FormFieldChoice* formFieldChoice = static_cast< Poppler::FormFieldChoice* >(formField);
+
+            if(formFieldChoice->choiceType() == Poppler::FormFieldChoice::ListBox || formFieldChoice->choiceType() == Poppler::FormFieldChoice::ComboBox)
+            {
+                formFields.append(new PdfFormField(m_mutex, formField));
+                continue;
+            }
+        }
+        else if(formField->type() == Poppler::FormField::FormButton)
+        {
+            Poppler::FormFieldButton* formFieldButton = static_cast< Poppler::FormFieldButton* >(formField);
+
+            if(formFieldButton->buttonType() == Poppler::FormFieldButton::CheckBox || formFieldButton->buttonType() == Poppler::FormFieldButton::Radio)
+            {
+                formFields.append(new PdfFormField(m_mutex, formField));
+                continue;
+            }
+        }
+
+        delete formField;
+    }
+
+    return formFields;
+}
+
+PdfDocument::PdfDocument(Poppler::Document* document) :
+    m_mutex(),
+    m_document(document)
+{
+}
+
+PdfDocument::~PdfDocument()
+{
+    delete m_document;
+}
+
+int PdfDocument::numberOfPages() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(&m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    return m_document->numPages();
+}
+
+Page* PdfDocument::page(int index) const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(&m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    Poppler::Page* page = m_document->page(index);
+
+    return page != 0 ? new PdfPage(&m_mutex, page) : 0;
+}
+
+bool PdfDocument::isLocked() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(&m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    return m_document->isLocked();
+}
+
+bool PdfDocument::unlock(const QString& password)
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(&m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    return m_document->unlock(password.toLatin1(), password.toLatin1());
+}
+
+QStringList PdfDocument::saveFilter() const
+{
+    return QStringList() << "Portable document format (*.pdf)";
+}
+
+bool PdfDocument::canSave() const
+{
+    return true;
+}
+
+bool PdfDocument::save(const QString& filePath, bool withChanges) const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(&m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    QScopedPointer< Poppler::PDFConverter > pdfConverter(m_document->pdfConverter());
+
+    pdfConverter->setOutputFileName(filePath);
+
+    if(withChanges)
+    {
+        pdfConverter->setPDFOptions(pdfConverter->pdfOptions() | Poppler::PDFConverter::WithChanges);
+    }
+
+    return pdfConverter->convert();
+}
+
+bool PdfDocument::canBePrintedUsingCUPS() const
+{
+    return true;
+}
+
+void PdfDocument::setPaperColor(const QColor& paperColor)
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(&m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    m_document->setPaperColor(paperColor);
+}
+
 void Model::PdfDocument::loadOutline(QStandardItemModel* outlineModel) const
 {
     Document::loadOutline(outlineModel);
@@ -767,7 +775,7 @@ void Model::PdfDocument::loadOutline(QStandardItemModel* outlineModel) const
     }
 }
 
-void Model::PdfDocument::loadProperties(QStandardItemModel* propertiesModel) const
+void PdfDocument::loadProperties(QStandardItemModel* propertiesModel) const
 {
     Document::loadProperties(propertiesModel);
 
@@ -797,7 +805,7 @@ void Model::PdfDocument::loadProperties(QStandardItemModel* propertiesModel) con
     }
 }
 
-void Model::PdfDocument::loadFonts(QStandardItemModel* fontsModel) const
+void PdfDocument::loadFonts(QStandardItemModel* fontsModel) const
 {
     Document::Document::loadFonts(fontsModel);
 
@@ -826,6 +834,78 @@ void Model::PdfDocument::loadFonts(QStandardItemModel* fontsModel) const
     }
 }
 
+bool PdfDocument::wantsContinuousMode() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(&m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    const Poppler::Document::PageLayout pageLayout = m_document->pageLayout();
+
+    return pageLayout == Poppler::Document::OneColumn
+        || pageLayout == Poppler::Document::TwoColumnLeft
+        || pageLayout == Poppler::Document::TwoColumnRight;
+}
+
+bool PdfDocument::wantsSinglePageMode() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(&m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    const Poppler::Document::PageLayout pageLayout = m_document->pageLayout();
+
+    return pageLayout == Poppler::Document::SinglePage
+        || pageLayout == Poppler::Document::OneColumn;
+}
+
+bool PdfDocument::wantsTwoPagesMode() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(&m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    const Poppler::Document::PageLayout pageLayout = m_document->pageLayout();
+
+    return pageLayout == Poppler::Document::TwoPageLeft
+        || pageLayout == Poppler::Document::TwoColumnLeft;
+}
+
+bool PdfDocument::wantsTwoPagesWithCoverPageMode() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(&m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    const Poppler::Document::PageLayout pageLayout = m_document->pageLayout();
+
+    return pageLayout == Poppler::Document::TwoPageRight
+        || pageLayout == Poppler::Document::TwoColumnRight;
+}
+
+bool PdfDocument::wantsRightToLeftMode() const
+{
+#ifdef HAS_POPPLER_26
+
+    return m_document->textDirection() == Qt::RightToLeft;
+
+#else
+
+    return false;
+
+#endif // HAS_POPPLER_26
+}
+
+} // Model
+
 PdfSettingsWidget::PdfSettingsWidget(QSettings* settings, QWidget* parent) : SettingsWidget(parent),
     m_settings(settings)
 {
@@ -888,6 +968,13 @@ PdfSettingsWidget::PdfSettingsWidget(QSettings* settings, QWidget* parent) : Set
     m_layout->addRow(tr("Thin line mode:"), m_thinLineModeComboBox);
 
 #endif // HAS_POPPLER_24
+
+    m_backendComboBox = new QComboBox(this);
+    m_backendComboBox->addItem(tr("Splash"));
+    m_backendComboBox->addItem(tr("Arthur"));
+    m_backendComboBox->setCurrentIndex(m_settings->value("backend", 0).toInt());
+
+    m_layout->addRow(tr("Backend:"), m_backendComboBox);
 }
 
 void PdfSettingsWidget::accept()
@@ -917,6 +1004,8 @@ void PdfSettingsWidget::accept()
     m_settings->setValue("thinLineMode", m_thinLineModeComboBox->currentIndex());
 
 #endif // HAS_POPPLER_24
+
+    m_settings->setValue("backend", m_backendComboBox->currentIndex());
 }
 
 void PdfSettingsWidget::reset()
@@ -945,6 +1034,8 @@ void PdfSettingsWidget::reset()
     m_thinLineModeComboBox->setCurrentIndex(0);
 
 #endif // HAS_POPPLER_24
+
+    m_backendComboBox->setCurrentIndex(0);
 }
 
 PdfPlugin::PdfPlugin(QObject* parent) : QObject(parent)
@@ -1013,6 +1104,17 @@ Model::Document* PdfPlugin::loadDocument(const QString& filePath) const
         }
 
 #endif // HAS_POPPLER_24
+
+        switch(m_settings->value("backend").toInt())
+        {
+        default:
+        case 0:
+            document->setRenderBackend(Poppler::Document::SplashBackend);
+            break;
+        case 1:
+            document->setRenderBackend(Poppler::Document::ArthurBackend);
+            break;
+        }
     }
 
     return document != 0 ? new Model::PdfDocument(document) : 0;
@@ -1023,8 +1125,10 @@ SettingsWidget* PdfPlugin::createSettingsWidget(QWidget* parent) const
     return new PdfSettingsWidget(m_settings, parent);
 }
 
+} // qpdfview
+
 #if QT_VERSION < QT_VERSION_CHECK(5,0,0)
 
-Q_EXPORT_PLUGIN2(qpdfview_pdf, PdfPlugin)
+Q_EXPORT_PLUGIN2(qpdfview_pdf, qpdfview::PdfPlugin)
 
 #endif // QT_VERSION
