@@ -41,6 +41,9 @@ class Page;
 
 #include "model.h"
 
+namespace qpdfview
+{
+
 class PdfPlugin;
 
 namespace Model
@@ -134,7 +137,7 @@ namespace Model
     {
         Q_DECLARE_TR_FUNCTIONS(Model::PdfDocument)
 
-        friend class ::PdfPlugin;
+        friend class qpdfview::PdfPlugin;
 
     public:
         ~PdfDocument();
@@ -157,8 +160,15 @@ namespace Model
 
         void loadOutline(QStandardItemModel* outlineModel) const;
         void loadProperties(QStandardItemModel* propertiesModel) const;
+
         void loadFonts(QStandardItemModel* fontsModel) const;
 
+        bool wantsContinuousMode() const;
+        bool wantsSinglePageMode() const;
+        bool wantsTwoPagesMode() const;
+        bool wantsTwoPagesWithCoverPageMode() const;
+        bool wantsRightToLeftMode() const;
+
     private:
         Q_DISABLE_COPY(PdfDocument)
 
@@ -212,12 +222,14 @@ private:
 
 #endif // HAS_POPPLER_24
 
+    QComboBox* m_backendComboBox;
+
 };
 
 class PdfPlugin : public QObject, Plugin
 {
     Q_OBJECT
-    Q_INTERFACES(Plugin)
+    Q_INTERFACES(qpdfview::Plugin)
 
 #if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
 
@@ -239,4 +251,6 @@ private:
 
 };
 
+} // qpdfview
+
 #endif // PDFMODEL_H
