@@ -30,6 +30,9 @@ class QFormLayout;
 class QLineEdit;
 class QPrinter;
 
+namespace qpdfview
+{
+
 struct PrintOptions;
 class Settings;
 
@@ -65,4 +68,6 @@ private:
 
 };
 
+} // qpdfview
+
 #endif // PRINTDIALOG_H
