@@ -25,6 +25,9 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "model.h"
 
+namespace qpdfview
+{
+
 RenderTask::RenderTask(QObject* parent) : QObject(parent), QRunnable(),
     m_isRunning(false),
     m_wasCanceled(false),
@@ -89,7 +92,7 @@ void RenderTask::run()
         break;
     }
 
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+#if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
 
     QImage image = m_page->render(m_devicePixelRatio * resolutionX, m_devicePixelRatio * resolutionY, m_rotation);
 
@@ -149,11 +152,13 @@ void RenderTask::cancel()
 
 void RenderTask::finish()
 {
+    emit finished();
+
     m_mutex.lock();
     m_isRunning = false;
     m_mutex.unlock();
 
     m_waitCondition.wakeAll();
-
-    emit finished();
 }
+
+} // qpdfview
