@@ -29,6 +29,9 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "global.h"
 
+namespace qpdfview
+{
+
 namespace Model
 {
 class Page;
@@ -83,4 +86,6 @@ private:
 
 };
 
+} // qpdfview
+
 #endif // RENDERTASK_H
