@@ -24,6 +24,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <QApplication>
 #include <QDesktopWidget>
+#include <QLocale>
 #include <QSettings>
 
 #if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
@@ -36,6 +37,26 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #endif // QT_VERSION
 
+namespace
+{
+
+inline QStringList trimmed(const QStringList& list)
+{
+    QStringList trimmedList;
+
+    foreach(const QString& item, list)
+    {
+        trimmedList.append(item.trimmed());
+    }
+
+    return trimmedList;
+}
+
+} // anonymous
+
+namespace qpdfview
+{
+
 Settings* Settings::s_instance = 0;
 
 Settings* Settings::instance()
@@ -199,14 +220,14 @@ Settings::PageItem::PageItem(QSettings* settings) :
 
 // presentation view
 
-bool Settings::PresentationView::sync() const
+bool Settings::PresentationView::synchronize() const
 {
-    return m_settings->value("presentationView/sync", Defaults::PresentationView::sync()).toBool();
+    return m_settings->value("presentationView/synchronize", Defaults::PresentationView::synchronize()).toBool();
 }
 
-void Settings::PresentationView::setSync(bool sync)
+void Settings::PresentationView::setSynchronize(bool synchronize)
 {
-    m_settings->setValue("presentationView/sync", sync);
+    m_settings->setValue("presentationView/synchronize", synchronize);
 }
 
 int Settings::PresentationView::screen() const
@@ -279,6 +300,11 @@ void Settings::DocumentView::setAutoRefresh(bool autoRefresh)
     m_settings->setValue("documentView/autoRefresh", autoRefresh);
 }
 
+int Settings::DocumentView::autoRefreshTimeout() const
+{
+    return m_settings->value("documentView/autoRefreshTimeout", Defaults::DocumentView::autoRefreshTimeout()).toInt();
+}
+
 void Settings::DocumentView::setPrefetch(bool prefetch)
 {
     m_prefetch = prefetch;
@@ -294,11 +320,6 @@ void Settings::DocumentView::setPrefetchDistance(int prefetchDistance)
     }
 }
 
-int Settings::DocumentView::autoRefreshTimeout() const
-{
-    return m_settings->value("documentView/autoRefreshTimeout", Defaults::DocumentView::autoRefreshTimeout()).toInt();
-}
-
 int Settings::DocumentView::prefetchTimeout() const
 {
     return m_settings->value("documentView/prefetchTimeout", Defaults::DocumentView::prefetchTimeout()).toInt();
@@ -437,6 +458,16 @@ void Settings::DocumentView::setLayoutMode(LayoutMode layoutMode)
     m_settings->setValue("documentView/layoutMode", static_cast< uint >(layoutMode));
 }
 
+bool Settings::DocumentView::rightToLeftMode() const
+{
+    return m_settings->value("documentView/rightToLeftMode", Defaults::DocumentView::rightToLeftMode()).toBool();
+}
+
+void Settings::DocumentView::setRightToLeftMode(bool rightToLeftMode)
+{
+    m_settings->setValue("documentView/rightToLeftMode", rightToLeftMode);
+}
+
 ScaleMode Settings::DocumentView::scaleMode() const
 {
     return static_cast< ScaleMode >(m_settings->value("documentView/scaleMode", static_cast< uint >(Defaults::DocumentView::scaleMode())).toUInt());
@@ -532,6 +563,26 @@ void Settings::MainWindow::setRecentlyUsed(const QStringList& recentlyUsed)
     m_settings->setValue("mainWindow/recentlyUsed", recentlyUsed);
 }
 
+bool Settings::MainWindow::keepRecentlyClosed() const
+{
+    return m_settings->value("mainWindow/keepRecentlyClosed", Defaults::MainWindow::keepRecentlyClosed()).toBool();
+}
+
+void Settings::MainWindow::setKeepRecentlyClosed(bool on)
+{
+    m_settings->setValue("mainWindow/keepRecentlyClosed", on);
+}
+
+int Settings::MainWindow::recentlyClosedCount() const
+{
+    return m_settings->value("mainWindow/recentlyClosedCount", Defaults::MainWindow::recentlyClosedCount()).toInt();
+}
+
+void Settings::MainWindow::setRecentlyClosedCount(int recentlyClosedCount)
+{
+    m_settings->setValue("mainWindow/recentlyClosedCount", recentlyClosedCount);
+}
+
 bool Settings::MainWindow::restoreTabs() const
 {
     return m_settings->value("mainWindow/restoreTabs", Defaults::MainWindow::restoreTabs()).toBool();
@@ -562,6 +613,11 @@ void Settings::MainWindow::setRestorePerFileSettings(bool on)
     m_settings->setValue("mainWindow/restorePerFileSettings", on);
 }
 
+int Settings::MainWindow::saveDatabaseInterval() const
+{
+    return m_settings->value("mainWindow/saveDatabaseInterval", Defaults::MainWindow::saveDatabaseInterval()).toInt();
+}
+
 int Settings::MainWindow::tabPosition() const
 {
     return m_settings->value("mainWindow/tabPosition", Defaults::MainWindow::tabPosition()).toInt();
@@ -588,6 +644,16 @@ void Settings::MainWindow::setTabVisibility(int tabVisibility)
     }
 }
 
+bool Settings::MainWindow::spreadTabs() const
+{
+    return m_settings->value("mainWindow/spreadTabs", Defaults::MainWindow::spreadTabs()).toBool();
+}
+
+void Settings::MainWindow::setSpreadTabs(bool spreadTabs)
+{
+    m_settings->setValue("mainWindow/spreadTabs", spreadTabs);
+}
+
 bool Settings::MainWindow::newTabNextToCurrentTab() const
 {
     return m_settings->value("mainWindow/newTabNextToCurrentTab", Defaults::MainWindow::newTabNextToCurrentTab()).toBool();
@@ -598,6 +664,16 @@ void Settings::MainWindow::setNewTabNextToCurrentTab(bool newTabNextToCurrentTab
     m_settings->setValue("mainWindow/newTabNextToCurrentTab", newTabNextToCurrentTab);
 }
 
+bool Settings::MainWindow::documentTitleAsTabTitle() const
+{
+    return m_settings->value("mainWindow/documentAsTabTitle", Defaults::MainWindow::documentTitleAsTabTitle()).toBool();
+}
+
+void Settings::MainWindow::setDocumentTitleAsTabTitle(bool documentTitleAsTabTitle)
+{
+    m_settings->setValue("mainWindow/documentAsTabTitle", documentTitleAsTabTitle);
+}
+
 bool Settings::MainWindow::currentPageInWindowTitle() const
 {
     return m_settings->value("mainWindow/currentPageInWindowTitle", Defaults::MainWindow::currentPageInWindowTitle()).toBool();
@@ -628,18 +704,6 @@ void Settings::MainWindow::setSynchronizeOutlineView(bool synchronizeOutlineView
     m_settings->setValue("mainWindow/synchronizeOutlineView", synchronizeOutlineView);
 }
 
-static QStringList trimmed(const QStringList& list)
-{
-    QStringList trimmedList;
-
-    foreach(const QString& item, list)
-    {
-        trimmedList.append(item.trimmed());
-    }
-
-    return trimmedList;
-}
-
 QStringList Settings::MainWindow::fileToolBar() const
 {
     return m_settings->value("mainWindow/fileToolBar", Defaults::MainWindow::fileToolBar()).toStringList();
@@ -907,6 +971,19 @@ Settings::Settings(QObject* parent) : QObject(parent),
 
 // defaults
 
+bool Defaults::DocumentView::rightToLeftMode()
+{
+#if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
+
+    return QLocale::system().textDirection() == Qt::RightToLeft;
+
+#else
+
+    return false;
+
+#endif // QT_VERSION
+}
+
 QString Defaults::MainWindow::path()
 {
 #if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
@@ -919,3 +996,5 @@ QString Defaults::MainWindow::path()
 
 #endif // QT_VERSION
 }
+
+} // qpdfview
