@@ -34,6 +34,9 @@ class QSettings;
 #include "global.h"
 #include "printoptions.h"
 
+namespace qpdfview
+{
+
 class Settings : public QObject
 {
     Q_OBJECT
@@ -124,8 +127,8 @@ public:
     class PresentationView
     {
     public:
-        bool sync() const;
-        void setSync(bool sync);
+        bool synchronize() const;
+        void setSynchronize(bool synchronize);
 
         int screen() const;
         void setScreen(int screen);
@@ -154,13 +157,14 @@ public:
         bool autoRefresh() const;
         void setAutoRefresh(bool autoRefresh);
 
+        int autoRefreshTimeout() const;
+
         inline bool prefetch() const { return m_prefetch; }
         void setPrefetch(bool prefetch);
 
         inline int prefetchDistance() const { return m_prefetchDistance; }
         void setPrefetchDistance(int prefetchDistance);
 
-        int autoRefreshTimeout() const;
         int prefetchTimeout() const;
 
         inline int pagesPerRow() const { return m_pagesPerRow; }
@@ -207,6 +211,9 @@ public:
         LayoutMode layoutMode() const;
         void setLayoutMode(LayoutMode layoutMode);
 
+        bool rightToLeftMode() const;
+        void setRightToLeftMode(bool rightToLeftMode);
+
         ScaleMode scaleMode() const;
         void setScaleMode(ScaleMode scaleMode);
 
@@ -257,6 +264,12 @@ public:
         QStringList recentlyUsed() const;
         void setRecentlyUsed(const QStringList& recentlyUsed);
 
+        bool keepRecentlyClosed() const;
+        void setKeepRecentlyClosed(bool on);
+
+        int recentlyClosedCount() const;
+        void setRecentlyClosedCount(int recentlyClosedCount);
+
         bool restoreTabs() const;
         void setRestoreTabs(bool on);
 
@@ -266,15 +279,23 @@ public:
         bool restorePerFileSettings() const;
         void setRestorePerFileSettings(bool on);
 
+        int saveDatabaseInterval() const;
+
         int tabPosition() const;
         void setTabPosition(int tabPosition);
 
         int tabVisibility() const;
         void setTabVisibility(int tabVisibility);
 
+        bool spreadTabs() const;
+        void setSpreadTabs(bool spreadTabs);
+
         bool newTabNextToCurrentTab() const;
         void setNewTabNextToCurrentTab(bool newTabNextToCurrentTab);
 
+        bool documentTitleAsTabTitle() const;
+        void setDocumentTitleAsTabTitle(bool documentTitleAsTabTitle);
+
         bool currentPageInWindowTitle() const;
         void setCurrentPageInWindowTitle(bool currentPageInWindowTitle);
 
@@ -429,7 +450,7 @@ public:
     class PresentationView
     {
     public:
-        static inline bool sync() { return false; }
+        static inline bool synchronize() { return false; }
         static inline int screen() { return -1; }
 
         static inline QColor backgroundColor() { return QColor(); }
@@ -446,10 +467,11 @@ public:
 
         static inline bool autoRefresh() { return false; }
 
+        static inline int autoRefreshTimeout() { return 750; }
+
         static inline bool prefetch() { return false; }
         static inline int prefetchDistance() { return 1; }
 
-        static inline int autoRefreshTimeout() { return 750; }
         static inline int prefetchTimeout() { return 250; }
 
         static inline int pagesPerRow() { return 3; }
@@ -482,6 +504,8 @@ public:
 
         static inline LayoutMode layoutMode() { return SinglePageMode; }
 
+        static bool rightToLeftMode();
+
         static inline ScaleMode scaleMode() { return ScaleFactorMode; }
 
         static inline qreal scaleFactor() { return 1.0; }
@@ -501,18 +525,26 @@ public:
     {
     public:
         static inline bool trackRecentlyUsed() { return false; }
-
         static inline int recentlyUsedCount() { return 10; }
 
+        static inline bool keepRecentlyClosed() { return false; }
+        static inline int recentlyClosedCount() { return 5; }
+
         static inline bool restoreTabs() { return false; }
         static inline bool restoreBookmarks() { return false; }
         static inline bool restorePerFileSettings() { return false; }
 
+        static inline int saveDatabaseInterval() { return 30000; }
+
         static inline int tabPosition() { return 0; }
         static inline int tabVisibility() { return 0; }
 
+        static inline bool spreadTabs() { return false; }
+
         static inline bool newTabNextToCurrentTab() { return true; }
 
+        static inline bool documentTitleAsTabTitle() { return true; }
+
         static inline bool currentPageInWindowTitle() { return false; }
         static inline bool instancfeNameInWindowTitle() { return false; }
 
@@ -559,4 +591,6 @@ private:
 
 };
 
+} // qpdfview
+
 #endif // SETTINGS_H
