@@ -39,18 +39,48 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include "documentview.h"
 #include "miscellaneous.h"
 
-static void setCurrentTextToColorName(QComboBox* comboBox, const QColor& color)
+namespace
+{
+
+using namespace qpdfview;
+
+void addSettingsWidget(QTabWidget* tabWidget, SettingsWidget*& settingsWidget, PluginHandler::FileType fileType)
+{
+    settingsWidget = PluginHandler::instance()->createSettingsWidget(fileType, tabWidget);
+
+    if(settingsWidget != 0)
+    {
+        tabWidget->addTab(settingsWidget, PluginHandler::fileTypeName(fileType));
+    }
+}
+
+void setCurrentTextToColorName(QComboBox* comboBox, const QColor& color)
 {
     comboBox->lineEdit()->setText(color.isValid() ? color.name() : QString());
 }
 
-static QColor getValidColorFromCurrentText(QComboBox* comboBox, const QColor& defaultColor)
+QColor getValidColorFromCurrentText(const QComboBox* comboBox, const QColor& defaultColor)
 {
     const QColor color(comboBox->currentText());
 
     return color.isValid() ? color : defaultColor;
 }
 
+void setCurrentIndexFromKeyboardModifiers(QComboBox* comboBox, const Qt::KeyboardModifiers& modifiers)
+{
+    comboBox->setCurrentIndex(comboBox->findData(static_cast< int >(modifiers)));
+}
+
+Qt::KeyboardModifier getKeyboardModifierFromItemData(const QComboBox* comboBox)
+{
+    return static_cast< Qt::KeyboardModifier >(comboBox->itemData(comboBox->currentIndex()).toInt());
+}
+
+} // anonymous
+
+namespace qpdfview
+{
+
 Settings* SettingsDialog::s_settings = 0;
 
 SettingsDialog::SettingsDialog(QWidget* parent) : QDialog(parent)
@@ -63,27 +93,9 @@ SettingsDialog::SettingsDialog(QWidget* parent) : QDialog(parent)
     m_graphicsTabWidget = new QTabWidget(this);
     m_graphicsTabWidget->addTab(new QWidget(this), tr("General"));
 
-#ifdef WITH_PDF
-
-    m_pdfSettingsWidget = PluginHandler::createPdfSettingsWidget(this);
-
-    if(m_pdfSettingsWidget != 0)
-    {
-        m_graphicsTabWidget->addTab(m_pdfSettingsWidget, "PDF");
-    }
-
-#endif // WITH_PDF
-
-#ifdef WITH_PS
-
-    m_psSettingsWidget = PluginHandler::createPsSettingsWidget(this);
-
-    if(m_psSettingsWidget != 0)
-    {
-        m_graphicsTabWidget->addTab(m_psSettingsWidget, "PS");
-    }
-
-#endif // WITH_PS
+    addSettingsWidget(m_graphicsTabWidget, m_pdfSettingsWidget, PluginHandler::PDF);
+    addSettingsWidget(m_graphicsTabWidget, m_psSettingsWidget, PluginHandler::PS);
+    addSettingsWidget(m_graphicsTabWidget, m_djvuSettingsWidget, PluginHandler::DjVu);
 
     m_graphicsLayout = new QFormLayout(m_graphicsTabWidget->widget(0));
 
@@ -146,106 +158,13 @@ void SettingsDialog::accept()
 {
     QDialog::accept();
 
-#ifdef WITH_PDF
+    acceptBehaivorTab();
 
-    if(m_pdfSettingsWidget != 0)
-    {
-        m_pdfSettingsWidget->accept();
-    }
+    acceptGraphicsTab();
 
-#endif // WITH_PDF
+    acceptInterfaceTab();
 
-#ifdef WITH_PS
-
-    if(m_psSettingsWidget != 0)
-    {
-        m_psSettingsWidget->accept();
-    }
-
-#endif // WITH_PS
-
-    // behavior
-
-    s_settings->documentView().setOpenUrl(m_openUrlCheckBox->isChecked());
-
-    s_settings->documentView().setAutoRefresh(m_autoRefreshCheckBox->isChecked());
-
-    s_settings->mainWindow().setTrackRecentlyUsed(m_trackRecentlyUsedCheckBox->isChecked());
-
-    s_settings->mainWindow().setRestoreTabs(m_restoreTabsCheckBox->isChecked());
-    s_settings->mainWindow().setRestoreBookmarks(m_restoreBookmarksCheckBox->isChecked());
-    s_settings->mainWindow().setRestorePerFileSettings(m_restorePerFileSettingsCheckBox->isChecked());
-
-    s_settings->presentationView().setSync(m_presentationSyncCheckBox->isChecked());
-    s_settings->presentationView().setScreen(m_presentationScreenSpinBox->value());
-
-    s_settings->documentView().setHighlightDuration(m_highlightDurationSpinBox->value());
-    s_settings->pageItem().setHighlightColor(getValidColorFromCurrentText(m_highlightColorComboBox, Defaults::PageItem::highlightColor()));
-    s_settings->pageItem().setAnnotationColor(getValidColorFromCurrentText(m_annotationColorComboBox, Defaults::PageItem::annotationColor()));
-
-    s_settings->documentView().setSourceEditor(m_sourceEditorLineEdit->text());
-
-    // graphics
-
-    s_settings->pageItem().setKeepObsoletePixmaps(m_keepObsoletePixmapsCheckBox->isChecked());
-
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
-
-    s_settings->pageItem().setUseDevicePixelRatio(m_useDevicePixelRatioCheckBox->isChecked());
-
-#endif // QT_VERSION
-
-    s_settings->pageItem().setDecoratePages(m_decoratePagesCheckBox->isChecked());
-    s_settings->pageItem().setDecorateLinks(m_decorateLinksCheckBox->isChecked());
-    s_settings->pageItem().setDecorateFormFields(m_decorateFormFieldsCheckBox->isChecked());
-
-    s_settings->pageItem().setBackgroundColor(getValidColorFromCurrentText(m_backgroundColorComboBox, Defaults::PageItem::backgroundColor()));
-    s_settings->pageItem().setPaperColor(getValidColorFromCurrentText(m_paperColorComboBox, Defaults::PageItem::paperColor()));
-    s_settings->presentationView().setBackgroundColor(getValidColorFromCurrentText(m_presentationBackgroundColorComboBox, Defaults::PresentationView::backgroundColor()));
-
-    s_settings->documentView().setPagesPerRow(m_pagesPerRowSpinBox->value());
-
-    s_settings->documentView().setPageSpacing(m_pageSpacingSpinBox->value());
-    s_settings->documentView().setThumbnailSpacing(m_thumbnailSpacingSpinBox->value());
-
-    s_settings->documentView().setThumbnailSize(m_thumbnailSizeSpinBox->value());
-
-    s_settings->pageItem().setCacheSize(m_cacheSizeComboBox->itemData(m_cacheSizeComboBox->currentIndex()).toInt());
-    s_settings->documentView().setPrefetch(m_prefetchCheckBox->isChecked());
-    s_settings->documentView().setPrefetchDistance(m_prefetchDistanceSpinBox->value());
-
-    // interface
-
-    s_settings->mainWindow().setTabPosition(m_tabPositionComboBox->itemData(m_tabPositionComboBox->currentIndex()).toInt());
-    s_settings->mainWindow().setTabVisibility(m_tabVisibilityComboBox->itemData(m_tabVisibilityComboBox->currentIndex()).toInt());
-
-    s_settings->mainWindow().setNewTabNextToCurrentTab(m_newTabNextToCurrentTabCheckBox->isChecked());
-
-    s_settings->mainWindow().setRecentlyUsedCount(m_recentlyUsedCountSpinBox->value());
-
-    s_settings->mainWindow().setFileToolBar(m_fileToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
-    s_settings->mainWindow().setEditToolBar(m_editToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
-    s_settings->mainWindow().setViewToolBar(m_viewToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
-
-    s_settings->mainWindow().setCurrentPageInWindowTitle(m_currentPageInWindowTitleCheckBox->isChecked());
-    s_settings->mainWindow().setInstanceNameInWindowTitle(m_instanceNameInWindowTitleCheckBox->isChecked());
-
-    s_settings->mainWindow().setSynchronizeOutlineView(m_synchronizeOutlineViewCheckBox->isChecked());
-
-    s_settings->documentView().setHighlightCurrentThumbnail(m_highlightCurrentThumbnailCheckBox->isChecked());
-    s_settings->documentView().setLimitThumbnailsToResults(m_limitThumbnailsToResultsCheckBox->isChecked());
-
-    s_settings->pageItem().setAnnotationOverlay(m_annotationOverlayCheckBox->isChecked());
-    s_settings->pageItem().setFormFieldOverlay(m_formFieldOverlayCheckBox);
-
-    // modifiers
-
-    s_settings->documentView().setZoomModifiers(static_cast<Qt::KeyboardModifier>(m_zoomModifiersComboBox->itemData(m_zoomModifiersComboBox->currentIndex()).toInt()));
-    s_settings->documentView().setRotateModifiers(static_cast<Qt::KeyboardModifier>(m_rotateModifiersComboBox->itemData(m_rotateModifiersComboBox->currentIndex()).toInt()));
-    s_settings->documentView().setScrollModifiers(static_cast<Qt::KeyboardModifier>(m_scrollModifiersComboBox->itemData(m_scrollModifiersComboBox->currentIndex()).toInt()));
-
-    s_settings->pageItem().setCopyToClipboardModifiers(static_cast<Qt::KeyboardModifier>(m_copyToClipboardModifiersComboBox->itemData(m_copyToClipboardModifiersComboBox->currentIndex()).toInt()));
-    s_settings->pageItem().setAddAnnotationModifiers(static_cast<Qt::KeyboardModifier>(m_addAnnotationModifiersComboBox->itemData(m_addAnnotationModifiersComboBox->currentIndex()).toInt()));
+    acceptModifiersTab();
 }
 
 void SettingsDialog::reset()
@@ -310,6 +229,14 @@ void SettingsDialog::createBehaviorTab()
 
     m_behaviorLayout->addRow(tr("Track recently used:"), m_trackRecentlyUsedCheckBox);
 
+    // keep recently closed
+
+    m_keepRecentlyClosedCheckBox = new QCheckBox(this);
+    m_keepRecentlyClosedCheckBox->setChecked(s_settings->mainWindow().keepRecentlyClosed());
+    m_keepRecentlyClosedCheckBox->setToolTip(tr("Effective after restart."));
+
+    m_behaviorLayout->addRow(tr("Keep recently closed:"), m_keepRecentlyClosedCheckBox);
+
     // restore tabs
 
     m_restoreTabsCheckBox = new QCheckBox(this);
@@ -339,12 +266,12 @@ void SettingsDialog::createBehaviorTab()
 
 #endif // WITH_SQL
 
-    // presentation sync
+    // synchronize presentation
 
-    m_presentationSyncCheckBox = new QCheckBox(this);
-    m_presentationSyncCheckBox->setChecked(s_settings->presentationView().sync());
+    m_synchronizePresentationCheckBox = new QCheckBox(this);
+    m_synchronizePresentationCheckBox->setChecked(s_settings->presentationView().synchronize());
 
-    m_behaviorLayout->addRow(tr("Synchronize presentation:"), m_presentationSyncCheckBox);
+    m_behaviorLayout->addRow(tr("Synchronize presentation:"), m_synchronizePresentationCheckBox);
 
     // presentation screen
 
@@ -387,6 +314,29 @@ void SettingsDialog::createBehaviorTab()
     m_behaviorLayout->addRow(tr("Source editor:"), m_sourceEditorLineEdit);
 }
 
+void SettingsDialog::acceptBehaivorTab()
+{
+    s_settings->documentView().setOpenUrl(m_openUrlCheckBox->isChecked());
+
+    s_settings->documentView().setAutoRefresh(m_autoRefreshCheckBox->isChecked());
+
+    s_settings->mainWindow().setTrackRecentlyUsed(m_trackRecentlyUsedCheckBox->isChecked());
+    s_settings->mainWindow().setKeepRecentlyClosed(m_keepRecentlyClosedCheckBox->isChecked());
+
+    s_settings->mainWindow().setRestoreTabs(m_restoreTabsCheckBox->isChecked());
+    s_settings->mainWindow().setRestoreBookmarks(m_restoreBookmarksCheckBox->isChecked());
+    s_settings->mainWindow().setRestorePerFileSettings(m_restorePerFileSettingsCheckBox->isChecked());
+
+    s_settings->presentationView().setSynchronize(m_synchronizePresentationCheckBox->isChecked());
+    s_settings->presentationView().setScreen(m_presentationScreenSpinBox->value());
+
+    s_settings->documentView().setHighlightDuration(m_highlightDurationSpinBox->value());
+    s_settings->pageItem().setHighlightColor(getValidColorFromCurrentText(m_highlightColorComboBox, Defaults::PageItem::highlightColor()));
+    s_settings->pageItem().setAnnotationColor(getValidColorFromCurrentText(m_annotationColorComboBox, Defaults::PageItem::annotationColor()));
+
+    s_settings->documentView().setSourceEditor(m_sourceEditorLineEdit->text());
+}
+
 void SettingsDialog::resetBehaviorTab()
 {
     m_openUrlCheckBox->setChecked(Defaults::DocumentView::openUrl());
@@ -394,12 +344,13 @@ void SettingsDialog::resetBehaviorTab()
     m_autoRefreshCheckBox->setChecked(Defaults::DocumentView::autoRefresh());
 
     m_trackRecentlyUsedCheckBox->setChecked(Defaults::MainWindow::trackRecentlyUsed());
+    m_keepRecentlyClosedCheckBox->setChecked(Defaults::MainWindow::keepRecentlyClosed());
 
     m_restoreTabsCheckBox->setChecked(Defaults::MainWindow::restoreTabs());
     m_restoreBookmarksCheckBox->setChecked(Defaults::MainWindow::restoreBookmarks());
     m_restorePerFileSettingsCheckBox->setChecked(Defaults::MainWindow::restorePerFileSettings());
 
-    m_presentationSyncCheckBox->setChecked(Defaults::PresentationView::sync());
+    m_synchronizePresentationCheckBox->setChecked(Defaults::PresentationView::synchronize());
     m_presentationScreenSpinBox->setValue(Defaults::PresentationView::screen());
 
     m_highlightDurationSpinBox->setValue(Defaults::DocumentView::highlightDuration());
@@ -418,7 +369,7 @@ void SettingsDialog::createGraphicsTab()
 
     m_graphicsLayout->addRow(tr("Keep obsolete pixmaps:"), m_keepObsoletePixmapsCheckBox);
 
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+#if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
 
     // use device pixel ratio
 
@@ -517,6 +468,8 @@ void SettingsDialog::createGraphicsTab()
     m_cacheSizeComboBox->addItem(tr("%1 MB").arg(128), 128 * 1024 * 1024);
     m_cacheSizeComboBox->addItem(tr("%1 MB").arg(256), 256 * 1024 * 1024);
     m_cacheSizeComboBox->addItem(tr("%1 MB").arg(512), 512 * 1024 * 1024);
+    m_cacheSizeComboBox->addItem(tr("%1 MB").arg(1024), 1073741823);
+    m_cacheSizeComboBox->addItem(tr("%1 MB").arg(2048), 2147483647);
     m_cacheSizeComboBox->setCurrentIndex(m_cacheSizeComboBox->findData(s_settings->pageItem().cacheSize()));
 
     m_graphicsLayout->addRow(tr("Cache size:"), m_cacheSizeComboBox);
@@ -537,29 +490,56 @@ void SettingsDialog::createGraphicsTab()
     m_graphicsLayout->addRow(tr("Prefetch distance:"), m_prefetchDistanceSpinBox);
 }
 
-void SettingsDialog::resetGraphicsTab()
+void SettingsDialog::acceptGraphicsTab()
 {
-#ifdef WITH_PDF
+    s_settings->pageItem().setKeepObsoletePixmaps(m_keepObsoletePixmapsCheckBox->isChecked());
+
+#if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
+
+    s_settings->pageItem().setUseDevicePixelRatio(m_useDevicePixelRatioCheckBox->isChecked());
+
+#endif // QT_VERSION
+
+    s_settings->pageItem().setDecoratePages(m_decoratePagesCheckBox->isChecked());
+    s_settings->pageItem().setDecorateLinks(m_decorateLinksCheckBox->isChecked());
+    s_settings->pageItem().setDecorateFormFields(m_decorateFormFieldsCheckBox->isChecked());
+
+    s_settings->pageItem().setBackgroundColor(getValidColorFromCurrentText(m_backgroundColorComboBox, Defaults::PageItem::backgroundColor()));
+    s_settings->pageItem().setPaperColor(getValidColorFromCurrentText(m_paperColorComboBox, Defaults::PageItem::paperColor()));
+    s_settings->presentationView().setBackgroundColor(getValidColorFromCurrentText(m_presentationBackgroundColorComboBox, Defaults::PresentationView::backgroundColor()));
+
+    s_settings->documentView().setPagesPerRow(m_pagesPerRowSpinBox->value());
+
+    s_settings->documentView().setPageSpacing(m_pageSpacingSpinBox->value());
+    s_settings->documentView().setThumbnailSpacing(m_thumbnailSpacingSpinBox->value());
+
+    s_settings->documentView().setThumbnailSize(m_thumbnailSizeSpinBox->value());
+
+    s_settings->pageItem().setCacheSize(m_cacheSizeComboBox->itemData(m_cacheSizeComboBox->currentIndex()).toInt());
+    s_settings->documentView().setPrefetch(m_prefetchCheckBox->isChecked());
+    s_settings->documentView().setPrefetchDistance(m_prefetchDistanceSpinBox->value());
 
     if(m_pdfSettingsWidget != 0)
     {
-        m_pdfSettingsWidget->reset();
+        m_pdfSettingsWidget->accept();
     }
 
-#endif // WITH_PDF
-
-#ifdef WITH_PS
-
     if(m_psSettingsWidget != 0)
     {
-        m_psSettingsWidget->reset();
+        m_psSettingsWidget->accept();
     }
 
-#endif // WITH_PS
+    if(m_djvuSettingsWidget != 0)
+    {
+        m_djvuSettingsWidget->accept();
+    }
+}
 
+void SettingsDialog::resetGraphicsTab()
+{
     m_keepObsoletePixmapsCheckBox->setChecked(Defaults::PageItem::keepObsoletePixmaps());
 
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+#if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
 
     m_useDevicePixelRatioCheckBox->setChecked(Defaults::PageItem::useDevicePixelRatio());
 
@@ -583,6 +563,21 @@ void SettingsDialog::resetGraphicsTab()
     m_cacheSizeComboBox->setCurrentIndex(m_cacheSizeComboBox->findData(Defaults::PageItem::cacheSize()));
     m_prefetchCheckBox->setChecked(Defaults::DocumentView::prefetch());
     m_prefetchDistanceSpinBox->setValue(Defaults::DocumentView::prefetchDistance());
+
+    if(m_pdfSettingsWidget != 0)
+    {
+        m_pdfSettingsWidget->reset();
+    }
+
+    if(m_psSettingsWidget != 0)
+    {
+        m_psSettingsWidget->reset();
+    }
+
+    if(m_djvuSettingsWidget != 0)
+    {
+        m_djvuSettingsWidget->reset();
+    }
 }
 
 void SettingsDialog::createInterfaceTab()
@@ -608,6 +603,13 @@ void SettingsDialog::createInterfaceTab()
 
     m_interfaceLayout->addRow(tr("Tab visibility:"), m_tabVisibilityComboBox);
 
+    // spread tabs
+
+    m_spreadTabsCheckBox = new QCheckBox(this);
+    m_spreadTabsCheckBox->setChecked(s_settings->mainWindow().spreadTabs());
+
+    m_interfaceLayout->addRow(tr("Spread tabs:"), m_spreadTabsCheckBox);
+
     // new tab next to current tab
 
     m_newTabNextToCurrentTabCheckBox = new QCheckBox(this);
@@ -624,6 +626,15 @@ void SettingsDialog::createInterfaceTab()
 
     m_interfaceLayout->addRow(tr("Recently used count:"), m_recentlyUsedCountSpinBox);
 
+    // recently closed count
+
+    m_recentlyClosedCountSpinBox = new QSpinBox(this);
+    m_recentlyClosedCountSpinBox->setRange(1, 25);
+    m_recentlyClosedCountSpinBox->setValue(s_settings->mainWindow().recentlyClosedCount());
+    m_recentlyClosedCountSpinBox->setToolTip(tr("Effective after restart."));
+
+    m_interfaceLayout->addRow(tr("Recently closed count:"), m_recentlyClosedCountSpinBox);
+
     // file tool bar
 
     m_fileToolBarLineEdit = new QLineEdit(this);
@@ -648,6 +659,13 @@ void SettingsDialog::createInterfaceTab()
 
     m_interfaceLayout->addRow(tr("View tool bar:"), m_viewToolBarLineEdit);
 
+    // document title as tab title
+
+    m_documentTitleAsTabTitleCheckBox = new QCheckBox(this);
+    m_documentTitleAsTabTitleCheckBox->setChecked(s_settings->mainWindow().documentTitleAsTabTitle());
+
+    m_interfaceLayout->addRow(tr("Document title as tab title:"), m_documentTitleAsTabTitleCheckBox);
+
     // current page in window title
 
     m_currentPageInWindowTitleCheckBox = new QCheckBox(this);
@@ -698,10 +716,40 @@ void SettingsDialog::createInterfaceTab()
     m_interfaceLayout->addRow(tr("Form field overlay:"), m_formFieldOverlayCheckBox);
 }
 
+void SettingsDialog::acceptInterfaceTab()
+{
+    s_settings->mainWindow().setTabPosition(m_tabPositionComboBox->itemData(m_tabPositionComboBox->currentIndex()).toInt());
+    s_settings->mainWindow().setTabVisibility(m_tabVisibilityComboBox->itemData(m_tabVisibilityComboBox->currentIndex()).toInt());
+    s_settings->mainWindow().setSpreadTabs(m_spreadTabsCheckBox->isChecked());
+
+    s_settings->mainWindow().setNewTabNextToCurrentTab(m_newTabNextToCurrentTabCheckBox->isChecked());
+
+    s_settings->mainWindow().setRecentlyUsedCount(m_recentlyUsedCountSpinBox->value());
+    s_settings->mainWindow().setRecentlyClosedCount(m_recentlyClosedCountSpinBox->value());
+
+    s_settings->mainWindow().setFileToolBar(m_fileToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
+    s_settings->mainWindow().setEditToolBar(m_editToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
+    s_settings->mainWindow().setViewToolBar(m_viewToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
+
+    s_settings->mainWindow().setDocumentTitleAsTabTitle(m_documentTitleAsTabTitleCheckBox->isChecked());
+
+    s_settings->mainWindow().setCurrentPageInWindowTitle(m_currentPageInWindowTitleCheckBox->isChecked());
+    s_settings->mainWindow().setInstanceNameInWindowTitle(m_instanceNameInWindowTitleCheckBox->isChecked());
+
+    s_settings->mainWindow().setSynchronizeOutlineView(m_synchronizeOutlineViewCheckBox->isChecked());
+
+    s_settings->documentView().setHighlightCurrentThumbnail(m_highlightCurrentThumbnailCheckBox->isChecked());
+    s_settings->documentView().setLimitThumbnailsToResults(m_limitThumbnailsToResultsCheckBox->isChecked());
+
+    s_settings->pageItem().setAnnotationOverlay(m_annotationOverlayCheckBox->isChecked());
+    s_settings->pageItem().setFormFieldOverlay(m_formFieldOverlayCheckBox);
+}
+
 void SettingsDialog::resetInterfaceTab()
 {
     m_tabPositionComboBox->setCurrentIndex(m_tabPositionComboBox->findData(static_cast< uint >(Defaults::MainWindow::tabPosition())));
     m_tabVisibilityComboBox->setCurrentIndex(m_tabVisibilityComboBox->findData(static_cast< uint >(Defaults::MainWindow::tabVisibility())));
+    m_spreadTabsCheckBox->setChecked(Defaults::MainWindow::spreadTabs());
 
     m_newTabNextToCurrentTabCheckBox->setChecked(Defaults::MainWindow::newTabNextToCurrentTab());
 
@@ -711,6 +759,8 @@ void SettingsDialog::resetInterfaceTab()
     m_editToolBarLineEdit->setText(Defaults::MainWindow::editToolBar().join(","));
     m_viewToolBarLineEdit->setText(Defaults::MainWindow::viewToolBar().join(","));
 
+    m_documentTitleAsTabTitleCheckBox->setChecked(Defaults::MainWindow::documentTitleAsTabTitle());
+
     m_currentPageInWindowTitleCheckBox->setChecked(Defaults::MainWindow::currentPageInWindowTitle());
     m_instanceNameInWindowTitleCheckBox->setChecked(Defaults::MainWindow::instancfeNameInWindowTitle());
 
@@ -756,14 +806,24 @@ void SettingsDialog::createModifiersTab()
     m_modifiersLayout->addRow(tr("Add annotation:"), m_addAnnotationModifiersComboBox);
 }
 
+void SettingsDialog::acceptModifiersTab()
+{
+    s_settings->documentView().setZoomModifiers(getKeyboardModifierFromItemData(m_zoomModifiersComboBox));
+    s_settings->documentView().setRotateModifiers(getKeyboardModifierFromItemData(m_rotateModifiersComboBox));
+    s_settings->documentView().setScrollModifiers(getKeyboardModifierFromItemData(m_scrollModifiersComboBox));
+
+    s_settings->pageItem().setCopyToClipboardModifiers(getKeyboardModifierFromItemData(m_copyToClipboardModifiersComboBox));
+    s_settings->pageItem().setAddAnnotationModifiers(getKeyboardModifierFromItemData(m_addAnnotationModifiersComboBox));
+}
+
 void SettingsDialog::resetModifiersTab()
 {
-    m_zoomModifiersComboBox->setCurrentIndex(m_zoomModifiersComboBox->findData(static_cast< int >(Defaults::DocumentView::zoomModifiers())));
-    m_rotateModifiersComboBox->setCurrentIndex(m_rotateModifiersComboBox->findData(static_cast< int >(Defaults::DocumentView::rotateModifiers())));
-    m_scrollModifiersComboBox->setCurrentIndex(m_scrollModifiersComboBox->findData(static_cast< int >(Defaults::DocumentView::scrollModifiers())));
+    setCurrentIndexFromKeyboardModifiers(m_zoomModifiersComboBox, Defaults::DocumentView::zoomModifiers());
+    setCurrentIndexFromKeyboardModifiers(m_rotateModifiersComboBox, Defaults::DocumentView::rotateModifiers());
+    setCurrentIndexFromKeyboardModifiers(m_scrollModifiersComboBox, Defaults::DocumentView::scrollModifiers());
 
-    m_copyToClipboardModifiersComboBox->setCurrentIndex(m_copyToClipboardModifiersComboBox->findData(static_cast< int >(Defaults::PageItem::copyToClipboardModifiers())));
-    m_addAnnotationModifiersComboBox->setCurrentIndex(m_addAnnotationModifiersComboBox->findData(static_cast< int >(Defaults::PageItem::addAnnotationModifiers())));
+    setCurrentIndexFromKeyboardModifiers(m_copyToClipboardModifiersComboBox, Defaults::PageItem::copyToClipboardModifiers());
+    setCurrentIndexFromKeyboardModifiers(m_addAnnotationModifiersComboBox, Defaults::PageItem::addAnnotationModifiers());
 }
 
 void SettingsDialog::createColorComboBox(QComboBox*& comboBox, const QColor& color)
@@ -785,5 +845,8 @@ void SettingsDialog::createModifiersComboBox(QComboBox*& comboBox, const Qt::Key
     comboBox->addItem(QShortcut::tr("Shift and Ctrl"), static_cast< int >(Qt::ShiftModifier | Qt::ControlModifier));
     comboBox->addItem(QShortcut::tr("Shift and Alt"), static_cast< int >(Qt::ShiftModifier | Qt::AltModifier));
     comboBox->addItem(QShortcut::tr("Ctrl and Alt"), static_cast< int >(Qt::ControlModifier | Qt::AltModifier));
-    comboBox->setCurrentIndex(comboBox->findData(static_cast< int >(modifiers)));
+
+    setCurrentIndexFromKeyboardModifiers(comboBox, modifiers);
 }
+
+} // qpdfview
