@@ -35,6 +35,9 @@ class QSpinBox;
 class QTableView;
 class QTabWidget;
 
+namespace qpdfview
+{
+
 class Settings;
 class SettingsWidget;
 
@@ -58,17 +61,9 @@ private:
     QTabWidget* m_graphicsTabWidget;
     QFormLayout* m_graphicsLayout;
 
-#ifdef WITH_PDF
-
     SettingsWidget* m_pdfSettingsWidget;
-
-#endif // WITH_PDF
-
-#ifdef WITH_PS
-
     SettingsWidget* m_psSettingsWidget;
-
-#endif // WITH_PS
+    SettingsWidget* m_djvuSettingsWidget;
 
     QTableView* m_shortcutsTableView;
 
@@ -88,12 +83,13 @@ private:
     QCheckBox* m_autoRefreshCheckBox;
 
     QCheckBox* m_trackRecentlyUsedCheckBox;
+    QCheckBox* m_keepRecentlyClosedCheckBox;
 
     QCheckBox* m_restoreTabsCheckBox;
     QCheckBox* m_restoreBookmarksCheckBox;
     QCheckBox* m_restorePerFileSettingsCheckBox;
 
-    QCheckBox* m_presentationSyncCheckBox;
+    QCheckBox* m_synchronizePresentationCheckBox;
     QSpinBox* m_presentationScreenSpinBox;
 
     QSpinBox* m_highlightDurationSpinBox;
@@ -103,13 +99,14 @@ private:
     QLineEdit* m_sourceEditorLineEdit;
 
     void createBehaviorTab();
+    void acceptBehaivorTab();
     void resetBehaviorTab();
 
     // graphics
 
     QCheckBox* m_keepObsoletePixmapsCheckBox;
 
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+#if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
 
     QCheckBox* m_useDevicePixelRatioCheckBox;
 
@@ -135,21 +132,26 @@ private:
     QSpinBox* m_prefetchDistanceSpinBox;
 
     void createGraphicsTab();
+    void acceptGraphicsTab();
     void resetGraphicsTab();
 
     // interface
 
     QComboBox* m_tabPositionComboBox;
     QComboBox* m_tabVisibilityComboBox;
+    QCheckBox* m_spreadTabsCheckBox;
 
     QCheckBox* m_newTabNextToCurrentTabCheckBox;
 
     QSpinBox* m_recentlyUsedCountSpinBox;
+    QSpinBox* m_recentlyClosedCountSpinBox;
 
     QLineEdit* m_fileToolBarLineEdit;
     QLineEdit* m_editToolBarLineEdit;
     QLineEdit* m_viewToolBarLineEdit;
 
+    QCheckBox* m_documentTitleAsTabTitleCheckBox;
+
     QCheckBox* m_currentPageInWindowTitleCheckBox;
     QCheckBox* m_instanceNameInWindowTitleCheckBox;
 
@@ -162,6 +164,7 @@ private:
     QCheckBox* m_formFieldOverlayCheckBox;
 
     void createInterfaceTab();
+    void acceptInterfaceTab();
     void resetInterfaceTab();
 
     // modifiers
@@ -174,6 +177,7 @@ private:
     QComboBox* m_addAnnotationModifiersComboBox;
 
     void createModifiersTab();
+    void acceptModifiersTab();
     void resetModifiersTab();
 
     // helper methods
@@ -183,4 +187,6 @@ private:
 
 };
 
+} // qpdfview
+
 #endif // SETTINGSDIALOG_H
